/*@(#)np.h	1.2.1.2          5/3/89*/
/*
 * Creation 1986 by Inge B. Kristiansen.
 *
 * 03/05/89 (phd+vfr) GLOBALs
 * xx/05/90 kk  version 2
 * xx/01/93 kk  added new type for DCE compatibility as well as ref types
 *              and allocate/freeing modes
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXSTR		100
#define streq(s, t)	(strcmp(s, t) == 0)

#ifdef lint
#define new(t) ((t *) 0)
#else
#define new(t) ((t *) calloc(1, sizeof(t)))
#endif

/*
 * Error message severity types.
 */
enum severity {
	WARNING,
	SERIOUS,
	FATAL
};

/*
 * Kinds of translation functions.
 */
enum translation {
	EXTERNALIZE,
	INTERNALIZE
};

/*
 * Cons cell for lisp operations.
 */
struct cons {
	struct cons *cdr_;
	struct cons *car_;
};
typedef struct cons *list;

#define NIL	((list) 0)
#define car(x)	((x)->car_)
#define cdr(x)	((x)->cdr_)

#define ONIL	((struct object *) 0)
#define ocar(x)	((struct object *) ((x)->car_))
#define ocdr(x)	((struct object *) ((x)->cdr_))

/*
 * Object classes.
 */
enum class {
	O_TYPE,
	O_CONSTANT,
	O_SYMBOL,
	O_TEXT
};

/*
 * Object class modifiers.
 */
enum cmod {
	O_NORMAL,
	O_ARG
};

/*
 * Type constructors.
 */
enum constr {
	C_PREDEF,
	C_ENUMERATION,
	C_ARRAY,
	C_SEQUENCE,
	C_RECORD,
	C_CHOICE,
	C_PROCEDURE,
	C_ERROR,
	C_EXECUTION,
	C_MESSAGE,
	C_FILE,
	C_MEMORY,
	C_INTERRUPT,
	C_GLOBAL,
	C_CONSTANT,
	C_INTERFACE,
	C_PRIVATE
};

/*
 * Type identifiers.
 */
/* The order must not be changed to not change checksum calculation !!! */

enum typeid {
	T_BOOLEAN,
	T_CARDINAL,
	T_LONG_CARDINAL,
	T_CHARACTER,
	T_INTEGER,
	T_LONG_INTEGER,
	T_FLOATING,
	T_LONG_FLOATING,
	T_STRING,
	T_UNSPECIFIED,
	T_LONG_UNSPECIFIED,
	T_COMPOSITE,
/*
	T_MEDIUM_CARDINAL,
	T_MEDIUM_INTEGER,
	T_MEDIUM_UNSPECIFIED
*/

	T_BYTE,
	T_SMALL,
	T_HYPER,
	T_USMALL,
	T_UHYPER,
	T_STRING_REF
};

/*
 * Allocate/free modes.
 */
#define FIRST_FREEING 0
#define LAST_FREEING 9
enum freeing {
	F_USER = FIRST_FREEING,
	F_AUTOFREE
};

/*
 * Addressing modes.
 */
enum addr {
	A_VALUE,
	A_REF
};

/*
 * Direction for parameter passing.
 */
enum dir {
	D_IN,
	D_OUT,
	D_INOUT
};

/*
 * Condition codes for parameter passing.
 */
enum parcondition {
	P_NP_ALL,
	P_SPECIFIED,
	P_CHECK
};

/*
 * Transmission structure, for address mode and direction.
 */
struct transmit {
	enum freeing t_freeing;         /* allocate/free mode */
	enum addr t_addr;               /* addressing mode */
	enum dir t_dir;                 /* direction */
};

/*
 * Object structure.
 *
 * This is used for types, numbers, and strings. It will also
 * hold the information for parameter transmission.
 */
struct object {
	enum class o_class;             /* object class */
	enum cmod o_cmod;               /* object class modifier */
	struct transmit *o_transmit;    /* transmission control structure */
	union {
		struct type *u_type;    /* type for O_TYPE */
		int u_value;            /* value for O_CONSTANT */
		char *u_name;           /* name for O_SYMBOL */
		char *u_text;           /* string for O_TEXT */
	} o_union;
};
#define o_freeing       o_transmit->t_freeing
#define o_addr          o_transmit->t_addr
#define o_dir           o_transmit->t_dir
#define o_type		o_union.u_type
#define o_value		o_union.u_value
#define o_name		o_union.u_name
#define o_text          o_union.u_text

/*
 * Type structure.
 *
 * Formats of various lists are as follows.
 * . indicates a cons operation.
 * ... indicates a list of elements of the preceding form.
 *
 * enumeration:
 *	((name . value) ...)
 * record, error, procedure arguments, procedure results:
 *	(((name ...) . type) ...)
 * choice:
 *	((((name . value) ...) . type) ...)
 */
struct type {
	enum constr type_constr;	/* constructor */
	enum typeid type_id;            /* type identifier */
	char *type_name;		/* name of C type */
	int type_xsize;			/* size of external form in 16-bit
					   words, or -1 if variable */
	union {
		list u_list;		/* enumeration, record, error */
		struct {
			struct object *u_size, *u_basetype;
		} u_array;		/* array, sequence */
		struct {
			struct object *u_designator;
			list u_candidates;
		} u_choice;		/* choice */
		struct {
			unsigned long u_typecheck;
			list u_args, u_results, u_errors, u_permission;
		} u_procedure;		/* procedure */
	} type_u;
};

/*
 * These definitions allow access from an object pointer known to be a type.
 */
#define t_constr	o_type->type_constr
#define t_id            o_type->type_id
#define t_name		o_type->type_name
#define t_xsize		o_type->type_xsize
#define t_list		o_type->type_u.u_list
#define t_size		o_type->type_u.u_array.u_size
#define t_basetype	o_type->type_u.u_array.u_basetype
#define t_designator	o_type->type_u.u_choice.u_designator
#define t_candidates	o_type->type_u.u_choice.u_candidates
#define t_typecheck     o_type->type_u.u_procedure.u_typecheck
#define t_args		o_type->type_u.u_procedure.u_args
#define t_results	o_type->type_u.u_procedure.u_results
#define t_errors	o_type->type_u.u_procedure.u_errors
#define t_permission    o_type->type_u.u_procedure.u_permission

/*
 * Predefined type structure.
 *
 * This structure will hold pointers to all the predefined types
 * known in the system.
 */
struct pretypes {
	struct object *pt_boolean_type;
	struct object *pt_byte_type;
	struct object *pt_character_type;

	struct object *pt_small_type;
	struct object *pt_integer_type;
	struct object *pt_long_integer_type;
	struct object *pt_hyper_type;

	struct object *pt_usmall_type;
	struct object *pt_cardinal_type;
	struct object *pt_long_cardinal_type;
	struct object *pt_uhyper_type;

	struct object *pt_floating_type;
	struct object *pt_long_floating_type;
	struct object *pt_string_type;
	struct object *pt_string_ref_type;
	struct object *pt_unspecified_type;
	struct object *pt_long_unspecified_type;
};
#define PT_NORMAL       0
#define PT_RAW          1
#define PT_ALTNORMAL    2
#define PT_ALTRAW       3
#define PT_MAXPREDEF    4

/*
 * Predefined types.
 */
extern struct pretypes *pt[ PT_MAXPREDEF ];
#define boolean_type            pt[PT_NORMAL]->pt_boolean_type
#define byte_type               pt[PT_NORMAL]->pt_byte_type
#define character_type          pt[PT_NORMAL]->pt_character_type

#define small_type              pt[PT_NORMAL]->pt_small_type
#define integer_type            pt[PT_NORMAL]->pt_integer_type
#define long_integer_type       pt[PT_NORMAL]->pt_long_integer_type
#define hyper_type              pt[PT_NORMAL]->pt_hyper_type

#define usmall_type             pt[PT_NORMAL]->pt_usmall_type
#define cardinal_type           pt[PT_NORMAL]->pt_cardinal_type
#define long_cardinal_type      pt[PT_NORMAL]->pt_long_cardinal_type
#define uhyper_type             pt[PT_NORMAL]->pt_uhyper_type

#define floating_type           pt[PT_NORMAL]->pt_floating_type
#define long_floating_type      pt[PT_NORMAL]->pt_long_floating_type
#define string_type             pt[PT_NORMAL]->pt_string_type
#define string_ref_type         pt[PT_NORMAL]->pt_string_ref_type
#define unspecified_type        pt[PT_NORMAL]->pt_unspecified_type
#define long_unspecified_type   pt[PT_NORMAL]->pt_long_unspecified_type

extern struct object
	*undefined_constant;

/*
 * Communication control.
 *
 * The following definitions will handle the communication control in
 * the system. This is connection establishment, directory servers,
 * loader servers etc.
 */

/*
 * Process binding modes.
 */
enum bind {
	B_FILE,                 /* Bind to given file name */
	B_FILE_PROCEDURE        /* Bind to given file and procedure */
};

/*
 * Import/export procedure prototype control enumeration.
 */
enum ie_conn {
	IE_SERVICE,             /* Service name */
	IE_HOSTNAME,            /* Host name parameter */
	IE_PORTNUMBER,          /* Port number variable */
	IE_PROTECTION,          /* Protection mode */
	IE_EXECUTION,           /* Execution mode */
	IE_LOAD,                /* Loader server mode */
	IE_FILENAME,            /* File name parameter */
	IE_DIRECTORY,           /* Directory mode */
	IE_DIRHOST,             /* Directory host */
	IE_DIRPORT,             /* Directory port */
	IE_PROTOCOL,            /* Protocol */
	IE_LOADPROTOCOL,        /* Load protocol */
	IE_DIRPROTOCOL,         /* Directory protocol */
	IE_TOABSOLUTE,          /* Connection life time */
	IE_TORELATIVE           /* No response timeout */
};

/*
 * Import/export procedure prototype object structure.
 *
 * This object is used in a table describing all that is needed to
 * define a parameter for the import/export calls.
 */
struct ie_object {
	enum ie_conn    ie_type;        /* Lookup code in table */
	char            *ie_name;       /* Symbol name */
	enum typeid     ie_typeid;      /* Type identifier */
	char            *ie_tname;      /* Type name if composite */
	enum addr       ie_amode;       /* Address mode */
};

/*
 * Client structure.
 *
 * This structure will hold all the information linked to the
 * client side of the binding. It contains several simple
 * attributes, and also listhead pointers for lists.
 */
struct client {
	enum bind       c_binding;      /* Constructor */
	list            c_regionbegin;  /* Critical region begin procedures */
	list            c_regionend;    /* Critical region end procedures */
	list            c_connection;   /* Import procedure list */
	char            *c_protection;  /* Protection scheme */
	char            *c_execution;   /* Execution scheme */
	char            *c_load;        /* Loader server */
	char            *c_directory;   /* Directory server */
	int             c_toabsolute;   /* Connection lifetime (0 is infinite) */
	int             c_torelative;   /* No response timeout */
};
#define NP_CBINDING     B_FILE
#define NP_CREGIONBEGIN NIL
#define NP_CREGIONEND   NIL
#define NP_CCONNECTION  NIL
#define NP_CPROTECTION  "NC_PR_PUBLIC"
#define NP_CEXECUTION   "NC_EX_MOTHER"
#define NP_CLOAD        "NC_L_NOLOAD"
#define NP_CDIRECTORY   "NC_D_NODIRECTORY"
#define NP_CTOABSOLUTE  60
#define NP_CTORELATIVE  30

/*
 * Server structure.
 *
 * This structure will hold all the information linked to the
 * server side of the binding. It contains several simple
 * attributes, and also listhead pointers for lists.
 */
struct server {
	list            s_regionbegin;  /* Critical region begin procedures */
	list            s_regionend;    /* Critical region end procedures */
	list            s_connection;   /* Export procedure list */
	char            *s_hostname;    /* Host name */
	int             s_portnumber;   /* Port number */
	char            *s_execfile;    /* Executable file name */
	char            *s_idffile;     /* Interface description file name */
	char            *s_parameters;  /* Server startup parameters */
	char            *s_protection;  /* Protection scheme */
	char            *s_execution;   /* Execution scheme */
	char            *s_policy;      /* Scheduling policy */
	int             s_maxconn;      /* Max number of connections */
	char            *s_load;        /* Loader server */
	char            *s_directory;   /* Directory server */
	int             s_toabsolute;   /* Connection lifetime (0 is infinite) */
	int             s_torelative;   /* Idle time for connection */
	unsigned short  s_permission;   /* Server permission flags */
};
#define NP_SREGIONBEGIN NIL
#define NP_SREGIONEND   NIL
#define NP_SCONNECTION  NIL
#define NP_SHOSTNAME    (char *)0
#define NP_SPORTNUMBER  0
#define NP_SEXECFILE    "NC_E_STRING"
#define NP_SIDFFILE     "NC_E_STRING"
#define NP_SPARAMETERS  "NC_E_STRING"
#define NP_SPROTECTION  "NC_PR_PUBLIC"
#define NP_SEXECUTION   "NC_EX_MOTHER"
#define NP_SPOLICY      "NC_SC_RROBIN"
#define NP_SMAXCONN     20
#define NP_SLOAD        "NC_L_NOLOAD"
#define NP_SDIRECTORY   "NC_D_NODIRECTORY"
#define NP_STOABSOLUTE  60
#define NP_STORELATIVE  30
#define NP_SPERMISSION  0755

/*
 * Mode control flag area definition.
 */
#define NP_F_STYPE      0170000         /* Type of server */
#define NP_F_SNORM      0010000         /* - Normal */
#define NP_F_SDIR       0020000         /* - Directory */

#define NP_F_ETYPE      0007000         /* Exclusion type */
#define NP_F_ENORM      0001000         /* - Normal */
#define NP_F_EMUTUAL    0002000         /* - Mutual exclusion */
#define NP_F_EEXTERN    0003000         /* - External */
#define NP_F_EMAXCONN   0004000         /* - Max number of connections */

#define NP_F_PUREAD     0000100         /* User read permission */
#define NP_F_PUWRITE    0000200         /* User write permission */

#define NP_F_PGREAD     0000010         /* Group read permission */
#define NP_F_PGWRITE    0000020         /* Group write permission */

#define NP_F_POREAD     0000001         /* Other read permission */
#define NP_F_POWRITE    0000002         /* Other write permission */

/*
 * Directory structure.
 *
 * This structure will hold all the information linked to the
 * directory side of the binding. It contains several simple
 * attributes, and also listhead pointers for lists.
 */
struct directory {
	char            *d_hostname;    /* Host name */
	int             d_portnumber;   /* Port number */
};
#define NP_DHOSTNAME    (char *)0
#define NP_DPORTNUMBER  800

/*
 * Transport protocol structure.
 *
 * This structure will hold all the information linked to the transport
 * protocol to be used for the communication. It contains several simple
 * attributes, and also listhead pointers for lists.
 */
struct transport {
	char            *tp_name;       /* Protocol name */
	char            *tp_load;       /* Load protocol name */
	char            *tp_directory;  /* Directory protocol name */
};
#define NP_TPNAME       "NC_P_RX"
#define NP_TPLOAD       "NC_P_NULL"
#define NP_TPDIRECTORY  "NC_P_NULL"

/*
 * Check attribute.
 *
 * This macro will check if an attribute is already updated
 * for one of the records above. It will display an error
 * message if the code is executed more than once.
 */
#define check_attribute( s )    { static short aflag = 0; check_flag( aflag++, s ); }

/*
 * Module handling.
 */
#define NP_DEF_MODULENAME       "unknown"

/*
 * Prefix handling.
 */
#define NP_DEF_PREFIXNAME       ""
#define NP_PREFIX_DELIMITER     "_"

/*
 * Nodal special grammar handling.
 */
#define NP_DATASERVER           ' '
#define NP_DOTINNAME            '.'


/*
 * External declarations.
 */
extern struct object *make_type(), *make_symbol(), *make_constant();
extern struct object *make_text();
extern struct object *lookup(), *basetype();
extern struct object *enumeration_type(), *constant_type();
extern struct object *array_type(), *sequence_type(), *nc_sequence_type();
extern struct object *record_type(), *choice_type();
extern struct object *private_type();
extern struct object *procedure_type(), *execution_type(), *error_type(),
                     *global_type();
extern struct object *message_type(), *file_type(), *memory_type();
extern struct object *interrupt_type();
extern struct object *get_predef(), *type_conv(), *type_fold();
extern list values, types, procedures, dependencies, connections;
extern list c_csymtab, c_ssymtab;
extern list cons(), nconc(), make_arg(), make_connection();
extern struct ie_object *get_connection();
extern struct transmit *make_transmission();
extern struct client c_client;
extern struct server c_server;
extern struct directory c_directory;
extern struct transport c_transport;
extern struct ie_object ie_table[];
extern FILE *header, *support1, *support2, *client, *server;
extern char *input_file, *module_name, *prefix_name;
extern char *copy(), *gensym(), *rep(), *repfree(), *refstr(), *derefstr(), *xfn();
extern int recons(), reconsfree();
extern char *make_prefix();
extern char *refamode(), *indamode();
extern enum addr chamode();
extern char *n_nameconv();
extern char *client_proc_name();
extern int client_proc_size();
extern unsigned short hash_service();
extern unsigned long hash_type();
extern int recursive_flag;

/*
 * Command processor.
 */
extern  int     np_isarg(), np_getargs();
extern  int     np_error();
extern  int     np_gen_client, np_gen_server, np_gen_prefix;
extern  char    *np_module;
extern  int     np_cflags(), np_alarms;

/*
 * Command processor defaults.
 */
#define NP_GEN_CLIENT   0
#define NP_GEN_SERVER   0
#define NP_GEN_PREFIX   0
#define NP_MODULE       NULL
#define NP_ALARMS       0
