/* %W% %E% */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
% ::                                                                        ::
% ::    MODULE:      Name         : NC_ARGTAB                               ::
% ::                 Author       : Inge Berge Kristiansen                  ::
% ::                 Version      : 1.20                                    ::
% ::                 Last updated : December 14th 1988          VFr             ::
% ::                                                                        ::
% ::    CONTENTS:    Standard command line processor definitions.           ::
% ::                                                                        ::
% ::    FUNCTION:    This module contain the definitions for the standard   ::
% ::                 command line processor getargs(). The command line     ::
% ::                 is scanned for all options starting with "-", leaving  ::
% ::                 the other arguments unchanged.                         ::
% ::                                                                        ::
% ::    ORIGIN:      Steve C. Johnson                                       ::
% ::                 American Telephone and Telegraph Company,              ::
% ::                 Murray Hill,                                           ::
% ::                 New Jersey, USA.                                       ::
% ::                                                                        ::
% ::    CONDITIONAL COMPILING:                                              ::
% ::    =================================================================   ::
% ::                                                                        ::
% ::                                                                        ::
% ::    REVISION LOG:                                                       ::
% ::    =================================================================   ::
% ::                                                                        ::
% ::    1.0          January 12th 1987                                      ::
% ::                 Initial version.                                       ::
% ::    1.20         14-12-88  VFr                                          ::
% ::                 modified version number                                ::
% ::    1.21         10-03-89  VFr                                          ::
% ::                 modified version number                                ::
% ::    1.22         28-04-89  VFr                                          ::
% ::                 modified version number                                ::
% ::    1.23         08-05-89  VFr                                          ::
% ::                 modified version number                                ::
% ::    1.31         27-09-89  VFr                                          ::
% ::                 modified version number                                ::
% ::    1.32         november 89  VFr                                       ::
% ::                 modified version number                                ::
% ::    1.33         december 89  VFr                                       ::
% ::    1.40         renamed nc.h to nccmp.h                                ::
% ::                 january  90  raw                                       ::
% ::    2.00         Version number 2.00                                    ::
% ::                 K. Kostro may/june 90                                  ::
% ::    2.1          Version number 2.1                                     ::
% ::                 K. Kostro Nov. 90                                      ::
% ::    2.2          Version number 2.2                                     ::
% ::                 K. Kostro started Dec. 90                              ::
% ::    2.4          Version number 2.4                                     ::
% ::                 K. Kostro installed March 91 on RT and 286             ::
% ::    2.5          Version number 2.5                                     ::
% ::                 K. Kostro installed April 91 (millisec timeout)        ::
% ::    2.6          Version number 2.6                                     ::
% ::                 K. Kostro started June 91 (NODAL access, broadcast RPC,::
% ::                                            all (!#@) bugs fixed)       ::
% ::    2.7          Version number 2.7                                     ::
% ::                 K. Kostro started Sept 91 ( )                          ::
% ::                                                                        ::
% ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

# define        NC_ARGTAB
# include       "sysconfig.h"
# include       "stdio.h"
# include       "getargs.h"
# include       "nccmp.h"


/*
 * Error messages.
 */

char    *nc_errnoarg    = "No such option -%c";
char    *nc_errnotav    = "Option -%c is not available in this version";


/*
 * The command processor header structure.
 */

static  GA_HEAD ga_head =
{
	"Usage: %s [options] filename(s)",
	"NC/RPC Front-End",
        '-',
        '.',
	nc_error,
	&nc_errnoarg,
	&nc_errnotav,
	NULL
};


/*
 * The command processor argument structure.
 */

static  GA_ARG  ga_argtab[] =
{
	{ 'C',  "",     GA_PROC,                (int *) nc_c_language,  "(C)         Generate C language network interface"                 },
	{ 'c',  "",     GA_COPY,                (int *) nc_np_append,   "(CLIENT)    Generate the client interface only"                    },
	{ 'D',  "sym=nn", GA_COPY,              (int *) nc_npp_append,  "(DEFINE)    Define preprocessor symbol like #define"               },
	{ 'f',  "flags",  GA_COPY | GA_SPACE,   (int *) nc_np_append,   "(FLAGS)     Set flags affecting compilation process"               },
	{ 'I',  "dir",  GA_COPY,                (int *) nc_npp_append,  "(INCLUDE)   Scan directory dir for include files"                  },
	{ 'n',  "",     GA_PROC,                (int *) nc_nodal,       "(NODAL)     Generate interface to Nodal processes"                 },
	{ 'o',  "name", GA_COPY | GA_SPACE,     (int *) nc_np_append,   "(MODULE)    Use the specified module name"                         },
	{ 'p',  "",     GA_COPY,                (int *) nc_np_append,   "(PREFIX)    Prefix procedures with service name"                   },
	{ 's',  "",     GA_COPY,                (int *) nc_np_append,   "(SERVER)    Generate the server interface only"                    },
	{ 'U',  "sym",  GA_COPY,                (int *) nc_npp_append,  "(UNDEFINE)  Undefine preprocessor symbol like #undef"              },
	{ 'v',  "",     GA_PROC,                (int *) nc_verbose,     "(VERBOSE)   Print programs called by nc"                 },
};


# define GA_TABSIZE ( sizeof( ga_argtab ) / sizeof( GA_ARG ) )


/*
 * Check if the argument is present on the command line.
 */

int
nc_isarg( argc, argv, option ) int argc; char *argv[]; char option;
{
	return( isarg( argc, argv, option ) );
}


/*
 * Get all the arguments at the command line.
 */

int
nc_getargs( argc, argv ) int argc; char *argv[];
{
	return( getargs( argc, argv, &ga_head, ga_argtab, GA_TABSIZE ) );
}
