/* %W% %E% */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
% ::                                                                        ::
% ::    MODULE:      Name         : NC_MAIN                                 ::
% ::                 Author       : Inge Berge Kristiansen                  ::
% ::                 Version      : 1.0                                     ::
% ::                 Last updated : January 12th 1987                       ::
% ::                                                                        ::
% ::    CONTENTS:    Main program module for the Network Compiler           ::
% ::                                                                        ::
% ::    FUNCTION:    This module contains the main program entry for        ::
% ::                 the Network Compiler. It's main task is to parse       ::
% ::                 the command line, build up the correct argument        ::
% ::                 lists for the preprocessor and the parser, and then    ::
% ::                 finally schedule the correct processes in the          ::
% ::                 correct sequence.                                      ::
% ::                                                                        ::
% ::    ORIGIN:      Steve C. Johnson                                       ::
% ::                 American Telephone and Telegraph Company,              ::
% ::                 Murray Hill,                                           ::
% ::                 New Jersey, USA.                                       ::
% ::                                                                        ::
% ::    CONDITIONAL COMPILING:                                              ::
% ::    =================================================================   ::
% ::                                                                        ::
% ::                                                                        ::
% ::    REVISION LOG:                                                       ::
% ::    =================================================================   ::
% ::                                                                        ::
% ::    1.0          January 12th 1987                                      ::
% ::                 Initial version.                                       ::
% ::                 January 08th 1990 (raw)                                ::
% ::                 renamed nc.h to nccmp.h                                ::
% ::                                                                        ::
% ::    26.09.90 kk     replaced tmpnam() by tempnam(). removed name alloc  ::
% ::    05.11.91 kk     Added check of NCHOME env variable. Made nc         ::
% ::                    returning correct exit status. Added verbose flag.  ::
% ::                                                                        ::
% ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

# define        NC_MAIN
# include       "sysconfig.h"
# include       "stdio.h"
# include       "nccmp.h"
# include       "nc_lan.h"

extern char *malloc();
extern char *tempnam();
extern void free();
char  *getenv(), nchome[256], *envval, *pa, *newpa[512], *strcat(), *strdup(), *strcpy();

/*
 * User definable global variables maintained by this module.
 */

nc_tlan nc_languages[] =                                /* Language array */
{
/* C */         NCC_PP, NCC_P,  NCC_CC, NCC_CN, NCC_CCFLAGS,    NCC_CNFLAGS,
/* MODULA2 */   NCM_PP, NCM_P,  NCM_CC, NCM_CN, NCM_CCFLAGS,    NCM_CNFLAGS,
/* FORTRAN */   NCF_PP, NCF_P,  NCF_CC, NCF_CN, NCF_CCFLAGS,    NCF_CNFLAGS,
/* ADA */       NCA_PP, NCA_P,  NCA_CC, NCA_CN, NCA_CCFLAGS,    NCA_CNFLAGS,
};

char    *nc_tmp =       NC_TMPFILE;                     /* Temporary file name */
int     nc_cls =        NC_DEFCLASS;                    /* Default class */
int     nc_lan =        NC_DEFLAN;                      /* Default language */
int     nc_verb =       0;                              /* Verbose option */


/*
 * System dependent global variables maintained by this module.
 */

char    *nc_npp;                                        /* Arguments to pp */
char    *nc_np;                                         /* Argumants to p */
char    *nc_ntmp;                                       /* Complete tmp file */
char    *nc_work;                                       /* Work area */


/*
 * Main entry point.
 */

int
main( argc, argv, envp ) int argc; char *argv[]; char *envp[];
{
	int status;
        /*
	 * C.S. usage of NCHOME is now depracated!
	 * 
	if ((envval = getenv("NCHOME")) != (char *)0 && strlen(envval) != 0) {
		strcpy(nchome, envval);
		strcat(nchome, "/bin/");
	} else
	*/
	
	nchome[0] = '\0';

	nc_npp = nc_alloc( "" );
	nc_np = nc_alloc( "" );
/*
	nc_ntmp = nc_alloc( nc_tmp );
*/
	nc_work = nc_alloc( "" );
	argc = nc_getargs( argc, argv );
	if (status = nc_files( argc, argv, envp ))
		exit(status);
	nc_free( nc_npp );
	nc_free( nc_np );
/*
	nc_free( nc_ntmp );
*/
	free( nc_ntmp );
	nc_free( nc_work );
	exit( 0 );
}


/*
 * Allocate space for an argument list.
 */

char *
nc_alloc( name ) char *name;
{
	char    *bp;

	bp = malloc( NC_ARGSIZE );
	strcpy( bp, name );
	strcat( bp, " " );
	return( bp );
}


/*
 * Free the space occupied by an argument list.
 */

void
nc_free( bp ) char *bp;
{
	free( bp );
}


/*
 * Append argument to the preprocessor argument list.
 */

int
nc_npp_append( string ) char *string;
{
	strcat( nc_npp, string );
	strcat( nc_npp, " " );
}


/*
 * Append argument to the parser argument list.
 */

int
nc_np_append( string ) char *string;
{
	strcat( nc_np, string );
	strcat( nc_np, " " );
}


/*
 * Set the language to C.
 */

int
nc_c_language( string ) char *string;
{
	nc_lan = NC_C;
}


/*
 * Set verbose option.
 */

int
nc_verbose( string ) char *string;
{
	nc_verb = 1;
}


/*
 * Set the language class to Nodal.
 */

int
nc_nodal( string ) char *string;
{
	nc_cls = NC_INTERPRETED;
}


/*
 * Process all the files in the argument list.
 */

int
nc_files( argc, argv, envp ) int argc; char *argv[]; char *envp[];
{
	int status;
	int     i;

#if defined(LYNX1p2)
	nc_ntmp = mktemp("/tmp/ncXXXXXX");
#else
	nc_ntmp = tempnam("/tmp", "nc");
#endif
	for( i = 1; i < argc; i++ )
	{
		if ((status = nc_compile( argv[ i ] )))
			return(status);
	}
	unlink( nc_ntmp );
	return(0);
}

/*
 * Compile a file.
 */

int
nc_compile( filename ) char *filename;
{
	int     status;

	status = nc_preprocess( filename );
	if( !status ) {
		status = nc_parse();
	}
	return(status);
/*
	if (status)
		exit(status);
*/
}


/*
 * Preprocess a file.
 */

int
nc_preprocess( filename ) char *filename;
{
	int status;

	strcpy( nc_work, nchome );
	strcat( nc_work, nc_languages[nc_lan].nc_preprocessor );
	strcat( nc_work, nc_npp );
	strcat( nc_work, nc_languages[nc_lan].nc_ppflags[nc_cls] );
	strcat( nc_work, " " );
	strcat( nc_work, filename );
	strcat( nc_work, " > " );
	strcat( nc_work, nc_ntmp );
	if (nc_verb)
		printf("nc: calling %s\n", nc_work);
/*
	if (status = system( nc_work )) {
		fprintf(stderr, "preprocessor failed: %s\n", nc_work);
		exit(1);
	}
*/
	if (status = system( nc_work )) {
		fprintf(stderr, "preprocessor failed: %s\n", nc_work);
	}
	return( status );
}


/*
 * Parse a file.
 */

int
nc_parse()
{
	int status;

	strcpy( nc_work, nchome );
	strcat( nc_work, nc_languages[nc_lan].nc_coder[nc_cls] );
	strcat( nc_work, nc_np );
	strcat( nc_work, nc_ntmp );
	if (nc_verb)
		printf("nc: calling %s\n", nc_work);
	if (status = system( nc_work )) {
		fprintf(stderr, "parser failed: %s\n", nc_work);
	}
	return( status );
}


/*
 * Error reporting routine for the Network Compiler.
 */

int
nc_error( prog_name, s, x, y ) char *prog_name; char *s;
{
	fprintf( stderr, prog_name );
	fprintf( stderr, ": 0: " );
	fprintf( stderr, s, x, y );
	fprintf( stderr, "\n" );
}
