/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 *
 * 04.07.90 kk  replaced begin/end critical by dummy procedures
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Generate functions to be executed at the entry to a stub,
 * thereby defining a critical region.
 */

client_begin_critical()
{
}


/*
 * Generate functions to be executed at the exit from a stub,
 * thereby defining a critical region.
 */

client_end_critical()
{
}


/*
 * Compute the length of the procedure name according to the binding
 * information in the client control block.
 */

int
client_proc_size( proc_name )
	char *proc_name;
{
	switch( c_client.c_binding ) {
		case B_FILE:
			return( 0 );

		case B_FILE_PROCEDURE:
			return( strlen( proc_name ) );

		default:
			error(FATAL, "internal error (client_proc_size): bad binding");
			/* NOTREACHED */
	}
}


/*
 * Compute the the procedure name according to the binding
 * information in the client control block.
 */

char *
client_proc_name( proc_name )
	char *proc_name;
{
	switch( c_client.c_binding ) {
		case B_FILE:
			return( "" );

		case B_FILE_PROCEDURE:
			return( proc_name );

		default:
			error(FATAL, "internal error (client_proc_name): bad binding");
			/* NOTREACHED */
	}
}


/*
 * Make the client import binding procedures according to what the user has
 * specified in the connection statements.
 */

client_import( lib_name, module_name, protocol_suffix )
	char *lib_name;
	char *module_name;
	char *protocol_suffix;
{
	list connlist;

	/* Loop through the list of connection procedures and generate  */
	/* them.                                                        */
	for( connlist = c_client.c_connection; connlist != NIL; connlist = cdr( connlist ) ) {
		/* Generate one connection procedure stub.              */
		client_oneimport( car( connlist ), lib_name, module_name, protocol_suffix );
	}
}


/*
 * Make one client import binding procedure according to what the user has
 * specified in the connection argument list for that procedure.
 */

client_oneimport( oneconn, lib_name, module_name, protocol_suffix )
	list oneconn;
	char *lib_name;
	char *module_name;
	char *protocol_suffix;
{
	struct object *name, *type;
	struct ie_object *ie;
	list connsymtab;
	int i;

	/* Get a pointer to the name of the procedure.                  */
	name = ocar( car( oneconn ) );

	/* Get a pointer to the type tree.                              */
	type = ocdr( car( oneconn ) );

	/* Get a pointer to the symbol table.                           */
	connsymtab = cdr( oneconn );

	/* Generate the procedure header according to the specs.        */
	gen_phead( client, "int", "", "", make_prefix( name->o_name ), module_name, type );
	fprintf( client,
"\tint service_entry_id;\n\n\
\t_%s_import = -1;\n",
		module_name );

	/* Begin critical section.                                      */
	client_begin_critical();
	fprintf( client,
"\tservice_entry_id = %s(", lib_name );

	/* Loop through the import/export table to find out if any of   */
	/* the parameters are specified.                                */
	for( i = 1, ie = ie_table; ie->ie_name != 0; ++i, ++ie ) {
		name = make_symbol( ie->ie_name );
		if( lookup( connsymtab, name ) == (struct object *)0 ) {
			switch( ie->ie_type ) {
				case IE_SERVICE:
					fprintf( client, "\"%s\", ", module_name );
					break;

				case IE_HOSTNAME:
					if( c_server.s_hostname ) {
						fprintf( client, "\"%s\", ", c_server.s_hostname );
					} else {
						fprintf( client, "%s, ", "NC_E_STRING" );
					}
					break;

				case IE_PORTNUMBER:
					fprintf( client, "%d, ", c_server.s_portnumber );
					break;

				case IE_PROTECTION:
					fprintf( client, "%s, ", c_client.c_protection );
					break;

				case IE_EXECUTION:
					fprintf( client, "%s, ", c_client.c_execution );
					break;

				case IE_LOAD:
					fprintf( client, "%s, ", c_client.c_load );
					break;

				case IE_FILENAME:
					fprintf( client, "%s, ", c_server.s_execfile );
					break;

				case IE_DIRECTORY:
					fprintf( client, "%s, ", c_client.c_directory );
					break;

				case IE_DIRHOST:
					if( c_directory.d_hostname ) {
						fprintf( client, "\"%s\", ", c_directory.d_hostname );
					} else {
						fprintf( client, "%s, ", "NC_E_STRING" );
					}
					break;

				case IE_DIRPORT:
					fprintf( client, "%d, ", c_directory.d_portnumber );
					break;

				case IE_PROTOCOL:
					fprintf( client, "%s%s, ", c_transport.tp_name, protocol_suffix );
					break;

				case IE_LOADPROTOCOL:
					fprintf( client, "%s%s, ", c_transport.tp_load, protocol_suffix );
					break;

				case IE_DIRPROTOCOL:
					fprintf( client, "%s%s, ", c_transport.tp_directory, protocol_suffix );
					break;

				case IE_TOABSOLUTE:
					fprintf( client, "(long)%d, ", c_client.c_toabsolute );
					break;

				case IE_TORELATIVE:
					fprintf( client, "(long)%d, ", c_client.c_torelative );
					break;

				default:
					error(FATAL, "illegal parameter class in client_import");
					/* NOTREACHED */
			}
		} else {
			fprintf( client, "%s, ", name->o_name );
		}

		/* Free the allocated space.                            */
		free( name->o_name );
		free( name );

		/* Format the output in a nice way.                     */
		if( i == 5 ) {
			fprintf( client, "\n\t\t" );
			i = 0;
		}
	}

	/* Complete the procedure call.                                 */
	fprintf( client, "0%o, %s, %s, %s, %d,\n\
\t\t%d, &_%s_import);\n",
		c_server.s_permission, c_server.s_idffile, c_server.s_parameters,
		c_server.s_policy, c_server.s_maxconn,
		hash_service( module_name ), module_name );

	/* End critical section.                                        */
	client_end_critical();

	/* Terminate the procedure.                                     */
	fprintf( client,
"\treturn (service_entry_id);\n\
}\n" );
}


/*
 * Make the client free binding procedure according to what the user has
 * specified in the connection argument list.
 */

client_free( lib_name, module_name )
	char *lib_name;
	char *module_name;
{
	/* Make the procedure header.                                   */
	fprintf(client,
"\n\
int\n\
free_%s(service_entry_id)\n\
\tint service_entry_id;\n\
{\n\
\tint status;\n\n",
	module_name);

	/* Begin critical section.                                      */
	client_begin_critical();
	fprintf(client,
"\tstatus = %s(service_entry_id);\n\
\t_%s_import = -1;\n",
		lib_name, module_name);

	/* End critical section.                                        */
	client_end_critical();
	fprintf(client,
"\treturn (status);\n\
}\n");
}


/*
 * Make the client bind procedure according to what the user has
 * specified in the connection argument list.
 */

client_bindto( lib_name, module_name )
	char *lib_name;
	char *module_name;
{
	/* Make the procedure header.                                   */
	fprintf(client,
"\n\
bind_%s(service_entry_id)\n\
\tint service_entry_id;\n\
{\n\tint prev_entry_id;\n\n\
\tprev_entry_id = _%s_import;\n\
\t_%s_import = service_entry_id;\n\
\treturn(prev_entry_id);\n\
}\n",
		module_name, module_name, module_name);
}
