/*@(#)cexec.c	1.2.1.2          5/8/89*/
/*
 * Creation 1986 by Inge B. Kristiansen.
 */
/*
 * 08/05/89 GLOBALs
 */

#include "sysconfig.h"
#include "np.h"

extern struct object *globallist;

/*
 * Generate the client stub for an execution call.
 */

client_execution(proc_name, type, proc_number, random_id)
	char *proc_name;
	struct object *type;
	int proc_number;
	long random_id;
{
	list p, q;
	int nresults, nerrors, fixed_size, variable_size;
	int first;
	struct object *t, *bt, *result_type;
	char *result_name, *ref, *rtname, *result_ref;
	int a_mode, d_mode;

	/*
	 * Make sure there is at most one result returned.
	 */
	nresults = length(type->t_results);
	if (nresults == 1)
		/*
		 * Could be multiple names with one type.
		 */
		nresults = length(car(car(type->t_results)));
	if (nresults > 1) {
		error(SERIOUS, "procedures that return multiple results are not supported");
		return;
	}
	if (nresults) {
		result_name = ocar(car(car(type->t_results)))->o_name;
		result_type = ocdr(car(type->t_results));
		rtname = rep(result_type);
	} else {
		rtname = "void";
	}

	/*
	 * Stub routine for client.
	 */
	if (nresults) {
		p = type->t_results;
		q = car(car(p));
		t = ocdr(car(p));
		result_ref = indamode(basetype(t), ocar(q)->o_addr);
	} else {
		result_ref = "";
	}

	extern_execution(rtname, result_ref, prefix_name, proc_name, "send", "recv");

	/*
	 * Generate the stub for sending.
	 */
	fprintf(client,
"\n\
long int\n\
%s%s_send(",
		prefix_name, proc_name);
	for (p = type->t_args, first = 1; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_OUT) {
			if ( !first )
				fprintf(client, ", ");
			for (q = car(car(p)); q != NIL; q = cdr(q)) {
				if (q != car(car(p)))
					fprintf(client, ", ");
				fprintf(client, "%s", ocar(q)->o_name);
			}
			first = 0;
		}
	}
	fprintf(client, ")\n");
	for (p = type->t_args; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_OUT) {
			t = ocdr(car(p));
			ref = indamode(basetype(t), ocar(q)->o_addr);
			for (q = car(car(p)); q != NIL; q = cdr(q))
				fprintf(client, "\t%s %s%s;\n", rep(t), ref, ocar(q)->o_name);
		}
	}
	fprintf(client, "{\n");
	fprintf(client,
"\t_btype *_buf;\n\
\tregister _btype *_bp;\n"
		);
	/*
	 * Determine the size of the arguments.
	 * This is like the code in record_type().
	 */
	fixed_size = 0;
	variable_size = 0;
	if (globallist != (struct object *)NULL) {
	    for (p = globallist->t_args; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_OUT) {
		    bt = basetype(ocdr(car(p)));
		    if (bt->t_xsize == -1) {
			variable_size = 1;
		    } else {
			/* count multiple arguments of the same type */
			fixed_size += length(car(car(p))) * bt->t_xsize;
		    }
		}
	    }
	}
	for (p = type->t_args; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_OUT) {
			bt = basetype(ocdr(car(p)));
			if (bt->t_xsize == -1) {
				variable_size = 1;
			} else {
				/* count multiple arguments of the same type */
				fixed_size += length(car(car(p))) * bt->t_xsize;
			}
		}
	}

	if (!variable_size) {
		/*
		 * The argument list is fixed-size.
		 */
		fprintf(client,
"\n\
\t_buf = nc_allocate(CALL_HEADER_BYTES + %d);\n",
			fixed_size);
	} else {
		/*
		 * There are some variable-size arguments.
		 */
		fprintf(client,
"\tregister int _n = %d;\n\
\n",
			fixed_size);
		if (globallist != (struct object *)NULL) {
		    for (p = globallist->t_args; p != NIL; p = cdr(p)) {
			q = car(car(p));
			if (ocar(q)->o_dir != D_OUT) {
			    t = ocdr(car(p));
			    bt = basetype(t);
			    if (bt->t_xsize != -1)
				continue;
			    /* ref = refstr(bt); */
			    ref = refamode(bt, ocar(q)->o_addr);
			    for (q = car(car(p)); q != NIL; q = cdr(q)) {
				fprintf(client,
					"\t_n += sizeof_%s(%s%s);\n",
					rep(t), ref, ocar(q)->o_name);
			    }
			}
		    }
		}
		for (p = type->t_args; p != NIL; p = cdr(p)) {
			q = car(car(p));
			if (ocar(q)->o_dir != D_OUT) {
				t = ocdr(car(p));
				bt = basetype(t);
				if (bt->t_xsize != -1)
					continue;
				/* ref = refstr(bt); */
				ref = refamode(bt, ocar(q)->o_addr);
				for (q = car(car(p)); q != NIL; q = cdr(q)) {
					fprintf(client,
"\t_n += sizeof_%s(%s%s);\n",
						rep(t), ref, ocar(q)->o_name);
				}
			}
		}
		fprintf(client,
"\t_buf = nc_allocate(CALL_HEADER_BYTES + _n);\n"
			);
	}
	fprintf(client,
"\t_bp = &_buf[CALL_HEADER_BYTES];\n"
		);

	/*
	 * Begin critical section.
	 */
	client_begin_critical();

	/*
	 * Generate code to externalize the arguments.
	 */
	if (globallist != (struct object *)NULL) {
	    gen_externalize( globallist, P_SPECIFIED );
	}
	gen_externalize( type, P_SPECIFIED );

	if (!variable_size) {
		fprintf(client,
"\treturn(nc_send_call(NCC_CALL, _%s_import, %d, %ldL, _buf, CALL_HEADER_BYTES + %d));\n",
			module_name, proc_number, random_id, fixed_size);
	} else {
		fprintf(client,
"\treturn(nc_send_call(NCC_CALL, _%s_import, %d, %ldL, _buf, CALL_HEADER_BYTES + _n));\n",
			module_name, proc_number, random_id);
	}
	fprintf(client, "}\n");

	/*
	 * Generate the stub for receiving.
	 */
	fprintf(client,
"\n\
%s %s\n\
%s%s_recv(",
		rtname, result_ref, prefix_name, proc_name);
	for (p = type->t_args, first = 1; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_IN) {
			if ( !first )
				fprintf(client, ", ");
			for (q = car(car(p)); q != NIL; q = cdr(q)) {
				if (q != car(car(p)))
					fprintf(client, ", ");
				fprintf(client, "%s", ocar(q)->o_name);
			}
			first = 0;
		}
	}
	fprintf(client, ")\n");
	for (p = type->t_args; p != NIL; p = cdr(p)) {
		q = car(car(p));
		if (ocar(q)->o_dir != D_IN) {
			t = ocdr(car(p));
			ref = indamode(basetype(t), ocar(q)->o_addr);
			for (q = car(car(p)); q != NIL; q = cdr(q))
				fprintf(client, "\t%s %s%s;\n", rep(t), ref, ocar(q)->o_name);
		}
	}
	fprintf(client, "{\n");
	if (nresults) {
		q = car(car(type->t_results));
		if (ocar(q)->o_addr == A_VALUE) {
#if defined(APOLLO10K)
		    /* floats and doubles are put into registers if not static */
		    if( (result_type->t_id == T_FLOATING) || (result_type->t_id == T_LONG_FLOATING)  )
			    fprintf(client, "\tstatic %s %s;\n", rtname, result_name);
		    else
#endif
			fprintf(client, "\t%s %s;\n", rtname, result_name);
		} else
			fprintf(client, "\tstatic %s %s;\n", rtname, result_name);
	}
	fprintf(client,
"\t_btype *_bp;\n\
\tregister _btype *_rp;\n\n"
		);

	fprintf(client,
"\tif((_rp =_bp = nc_get_reply(NCC_RETURN, _%s_import, (char *)0, 0)) == 0)\n",
			module_name);
	/* check the results and return error or set exception */
	gen_error(client, type, rtname, result_ref, nresults);

	/*
	 * Generate code to internalize the results.
	 */
	gen_results( type, nresults, result_type, result_name );

	/*
	 * Generate code to internalize the OUT arguments.
	 */
	if (globallist != (struct object *)NULL) {
	    gen_internalize( globallist, P_SPECIFIED );
	}
	gen_internalize( type, P_SPECIFIED );

	/*
	 * End critical section.
	 */
	client_end_critical();

	/*
	 * Deallocate the buffer and return the return value if any.
	 */
	 fprintf(client, "\tnc_deallocate(_bp); /* allocated by nc_call */\n");
	if (nresults) {
		p = type->t_results;
		q = car(car(p));
		t = ocdr(car(p));
		ref = refamode(basetype(t), chamode(ocar(q)->o_addr));
		fprintf(client, "\treturn (%s%s);\n}\n",
			ref, result_name);
	} else {
		fprintf(client, "}\n");
	}
}
