/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Generate the server stub for a file access call.
 */

server_file_access(proc_name, type, proc_number, random_id)
	char *proc_name;
	struct object *type;
	int proc_number;
	long random_id;
{
	list p, q;
	int nresults, fixed_size, variable_size;
	struct object *t, *bt, *result_type;
	char *result_name, *ref, *rtname, *result_ref;
	int a_mode, d_mode;

	/*
	 * Make sure there is at most one result returned.
	 */
	nresults = length(type->t_results);
	if (nresults == 1)
		/*
		 * Could be multiple names with one type.
		 */
		nresults = length(car(car(type->t_results)));
	if (nresults > 1) {
		error(SERIOUS, "procedures that return multiple results are not supported");
		return;
	}
	if (nresults) {
		result_name = ocar(car(car(type->t_results)))->o_name;
		result_type = ocdr(car(type->t_results));
		rtname = rep(result_type);
	} else {
		rtname = "void";
	}

	/*
	 * Server routine.
	 */
	if (nresults) {
		p = type->t_results;
		q = car(car(p));
		t = ocdr(car(p));
		result_ref = indamode(basetype(t), ocar(q)->o_addr);
	} else {
		result_ref = "";
	}

	/*
	 * Generate the stub for writing.
	 */
	fprintf(server,
"\nserver_%s_write(_bp, _result_base, _result_len)\n\
\t_btype *_bp;\n\
\t_btype **_result_base;\n\
\tint *_result_len;\n\
{\n\
\tregister int _n;\n\
\tregister long_cardinal _size;\n",
		proc_name);

	/*
	 * There is a file descriptor attached to the data.
	 */
	p = type->t_args;
	t = ocdr(car(p));
	q = car(car(p));
	fprintf(server, "\t%s %s;\n", rep(t), ocar(q)->o_name);
	fprintf(server, "\n");

	/*
	 * Begin critical section.
	 */
	server_end_critical();

	/*
	 * Generate code to internalize the file descriptor.
	 */
	p = type->t_args;
	t = ocdr(car(p));
	q = car(car(p));
	ref = refstr(basetype(t));
	fprintf(server, "\t_bp += %s(%s%s, _bp);\n",
		xfn(INTERNALIZE, t), ref, ocar(q)->o_name);

	/*
	 * Generate code to write the data to the file.
	 */
	fprintf(server,
"\t_bp -= sizeof(idl_long_int);\n\
\t_n = *_result_len - sizeof(idl_ushort_int) + sizeof(idl_ulong_int);\n\
\t_size = (long_cardinal)_n;\n\
\texternalize_long_cardinal(&_size, _bp);\n\
\twrite(%s, _bp, _n);\n\
\t*_result_len = RETURN_HEADER_BYTES;\n\
\t_bp = nc_allocate(RETURN_HEADER_BYTES);\n\
\t*_result_base = _bp;\n",
		ocar(q)->o_name);

	/*
	 * End critical section.
	 */
	server_begin_critical();

	fprintf(server,
"}\n"
		);

	/*
	 * Generate the stub for reading.
	 */
	fprintf(server,
"\nserver_%s_read(_bp, _result_base, _result_len)\n\
\tregister _btype *_bp;\n\
\t_btype **_result_base;\n\
\tint *_result_len;\n\
{\n\
\tregister int _n;\n\
\tregister long_cardinal _size;\n\
\tchar _size_buffer[sizeof(idl_ulong_int)];\n",
		proc_name);

	/*
	 * There is a file descriptor attached to the data.
	 */
	p = type->t_args;
	t = ocdr(car(p));
	q = car(car(p));
	fprintf(server, "\t%s %s;\n", rep(t), ocar(q)->o_name);
	fprintf(server, "\n");

	/*
	 * Begin critical section.
	 */
	server_end_critical();

	/*
	 * Generate code to internalize the file descriptor.
	 */
	p = type->t_args;
	t = ocdr(car(p));
	q = car(car(p));
	ref = refstr(basetype(t));
	fprintf(server, "\t_bp += %s(%s%s, _bp);\n",
		xfn(INTERNALIZE, t), ref, ocar(q)->o_name);

	/*
	 * Generate code to read the data from the file.
	 */
	fprintf(server,
"\tread(%s, _size_buffer, sizeof(idl_ulong_int));\n\
\tinternalize_long_cardinal(&_size, _size_buffer);\n\
\t_n = (int)_size - sizeof(idl_ulong_int) + sizeof(idl_ushort_int);\n",
		ocar(q)->o_name);

	fprintf(server,
"\t*_result_len = RETURN_HEADER_BYTES + _n;\n\
\t_bp = nc_allocate(RETURN_HEADER_BYTES + _n);\n\
\t*_result_base = _bp;\n\
\t_bp = &_bp[RETURN_HEADER_BYTES];\n"
		);

	/*
	 * Generate code to externalize the file descriptor.
	 */
	p = type->t_args;
	t = ocdr(car(p));
	ref = refstr(basetype(t));
	q = car(car(p));
	fprintf(server, "\t_bp += %s(%s%s, _bp);\n",
		xfn(EXTERNALIZE, t),
		ref, ocar(q)->o_name);

	fprintf(server,
"\tread(%s, _bp, _n - sizeof(idl_ushort_int));\n",
		ocar(q)->o_name);

	/*
	 * End critical section.
	 */
	server_begin_critical();

	fprintf(server,
"}\n"
		);
}
