/*%W%          %G%*/

/*

ORIGIN:      Steve C. Johnson
	     American Telephone and Telegraph Company,
	     Murray Hill,
	     New Jersey, USA.

CONTENTS:    Standard command line processor definitions.

FUNCTION:    This module contain the definitions for the standard
	     command line processor getargs(). The command line
	     is scanned for all options starting with "-", leaving
	     the other arguments unchanged.


REVISION LOG:
=================================================================

*/


# define        NP_ARGTAB
# include       "sysconfig.h"
# include       "stdio.h"
# include       "getargs.h"
# include       "np.h"


/*
 * Command processor switch flags.
 */

int     np_gen_client           = NP_GEN_CLIENT;        /* Generate client code */
int     np_gen_server           = NP_GEN_SERVER;        /* Generate server code */
int     np_gen_prefix           = NP_GEN_PREFIX;        /* Generate procedure name prefix */
char    *np_module              = NP_MODULE;            /* Module name */


/*
 * Error messages.
 */

char    *np_errnoarg    = "No such option -%c";
char    *np_errnotav    = "Option -%c is not available in this version";


/*
 * The command processor header structure.
 */

static  GA_HEAD ga_head =
{
	"Usage: %s [options] filename",
	"Network Compiler Version %d.%d",
        '-',
        '.',
	np_error,
	&np_errnoarg,
	&np_errnotav,
	NULL
};


/*
 * The command processor argument structure.
 */

static  GA_ARG  ga_argtab[] =
{
	{ 'c',  "",     GA_BOOLEAN,             &np_gen_client,         "(CLIENT)    Generate the client interface only"                    },
	{ 'f',  "flags",GA_PROC | GA_SPACE,     (int *)np_cflags,       "(FLAGS)     Set flags affecting compilation process"               },
	{ 'o',  "name", GA_STRING | GA_SPACE,   (int *)&np_module,      "(MODULE)    Use the specified module name"                         },
	{ 'p',  "",     GA_BOOLEAN,             &np_gen_prefix,         "(PREFIX)    Prefix procedures with service name"                   },
	{ 's',  "",     GA_BOOLEAN,             &np_gen_server,         "(SERVER)    Generate the server interface only"                    },
};


# define GA_TABSIZE ( sizeof( ga_argtab ) / sizeof( GA_ARG ) )


/*
 * Check if the argument is present on the command line.
 */

int
np_isarg( argc, argv, option ) int argc; char *argv[]; char option;
{
	return( isarg( argc, argv, option ) );
}


/*
 * Get all the arguments at the command line.
 */

int
np_getargs( argc, argv ) int argc; char *argv[];
{
	return( getargs( argc, argv, &ga_head, ga_argtab, GA_TABSIZE ) );
}
