/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 *
 * xx.05.90 kk  Version 2.0
 * 05.06.90 kk  removed file from the default connection information
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Global data structures maintained by this module.
 */

struct client   c_client = {            /* Client control block */
		NP_CBINDING,            /* Default client binding */
		NP_CREGIONBEGIN,        /* Default region begin */
		NP_CREGIONEND,          /* Default region end */
		NP_CCONNECTION,         /* Default import mode */
		NP_CPROTECTION,         /* Default protection */
		NP_CEXECUTION,          /* Default execution mode */
		NP_CLOAD,               /* Default load mode */
		NP_CDIRECTORY,          /* Default directory */
		NP_CTOABSOLUTE,         /* Default connection lifetime */
		NP_CTORELATIVE          /* Default no response timeout */
};

static char     *c_string = "client";   /* Used for error reporting */


/*
 * Set the client binding.
 */

client_bind( bind_mode )
	enum bind bind_mode;
{
	check_attribute( c_string );
	c_client.c_binding = bind_mode;
}


/*
 * Set the client region BEGIN information.
 */

client_regionbegin( region )
	list region;
{
	check_attribute( c_string );
	c_client.c_regionbegin = region;
}


/*
 * Set the client region END information.
 */

client_regionend( region )
	list region;
{
	check_attribute( c_string );
	c_client.c_regionend = region;
}

/*
 * Initialize the client connection information. This procedure is
 * called before the parsing starts.
 */

client_initconnection()
{
	list arglist;
	list importlist;
	list connlist;

	arglist = cons( (list)make_connection( IE_HOSTNAME ), NIL );
/* 05.06.90 kk remove definitely later
	arglist = nconc( arglist, cons( (list)make_connection( IE_FILENAME ), NIL ) );
*/
	arglist = nconc( arglist, cons( (list)make_connection( IE_PORTNUMBER ), NIL ) );
	importlist = cons( make_symbol("import"), (list)procedure_type( (list)arglist, NIL, NIL ) );
	connlist = cons( cons( importlist, connections ), NIL );
	c_client.c_connection = connlist;
	connections = NIL;
}

/*
 * Set the client connection information.
 */

client_connection( importlist )
	list importlist;
{
	list connlist;
	static short aflag = 0;

	if( !aflag ) {
		++aflag;
		connlist = cons( cons( importlist, connections ), NIL );
	} else {
		connlist = c_client.c_connection;
		connlist = nconc( connlist, cons( cons( importlist, connections ), NIL ) );
	}
	c_client.c_connection = connlist;
	connections = NIL;
}

/*
 * Set the client protection information.
 */

client_protection( protection )
	struct object *protection;
{
	check_attribute( c_string );
	c_client.c_protection = repfree( protection );
}

/*
 * Set the client execution information.
 */

client_execmode( execution )
	struct object *execution;
{
	check_attribute( c_string );
	c_client.c_execution = repfree( execution );
}

/*
 * Set the client load information.
 */

client_load( load )
	struct object *load;
{
	check_attribute( c_string );
	c_client.c_load = repfree( load );
}

/*
 * Set the client directory information.
 */

client_directory( directory )
	struct object *directory;
{
	check_attribute( c_string );
	c_client.c_directory = repfree( directory );
}

/*
 * Set the connection life time.
 */

client_toabsolute( number )
	struct object *number;
{
	check_attribute( c_string );
	c_client.c_toabsolute = reconsfree( number );
}

/*
 * Set the no response timeout.
 */

client_torelative( number )
	struct object *number;
{
	check_attribute( c_string );
	c_client.c_torelative = reconsfree( number );
}
