%{
/*@(#)grammar.y	1.2.1.2          5/3/89*/
/*
 * Creation 1986 by Inge B. Kristiansen.
 *
 * 03/05/89 (phd+vfr) GLOBALs
 * 12.11.92 kk  added private, sequence_ref, string_ref, autofree
 * 22.01.93 kk  removed medium, added nc_sequence
 */

#include "sysconfig.h"                                  /* Configuration */
#include "np.h"                                         /* Common includes */
#undef  t_name                                          /* Used by some yacc's */
%}

%token
	identifier
	number
	atext
	permission

%token
	ABSOLUTE
	ADDRESS
	ARRAY
	AUTHOR
	AUTOFREE
	BEGIN_
	BINDING
	BOOLEAN
	BY
	BYTE
	CARDINAL
	CHAR
	CHOICE
	CLIENT
	CONNECTION
	CONSTANT
	DATE
	DEPENDS
	DIRECTORY
	DOUBLE
	END
	ERROR
	EXCEPTION
	EXECUTABLE
	EXECUTION
	FILE_
	FLOAT
	GLOBAL
	HOST
	HYPER
	IN
	INOUT
	INTEGER
	INTERFACE
	INTERRUPT
	LOAD
	LONG
	MAXCONNECTIONS
	MEMORY
	MESSAGE
	MODIFIED
	NAME
	OF
	OUT
	PARAMETERS
	PERMISSION
	POLICY
	PORT
	PRIVATE
	PROCEDURE
	PROGRAM
	PROTECTION
	PROTOCOL
	READ
	RECORD
	REF
	REGION
	RELATIVE
	RETURNS
	SERVER
	SERVICE
	SEQUENCE
	SEQUENCE_REF
	NC_SEQUENCE
	NC_SEQUENCE_REF
	SHORT
	SMALL
	STATISTICS
	STRING
	STRING_REF
	TIMEOUT
	TYPE
	UNSIGNED
	UNSPECIFIED
	UPON
	VALUE
	VERSION
	WRITE

%union {
	int integer;
	list list;
	struct object *object;
	struct transmit *transmit;
	enum freeing free_mode;
	enum addr a_mode;
	enum dir d_mode;
	enum bind bind_mode;
}

%type <object>
	access_type
	constructed_type
	designator_type
	identifier
	maximum_number
	module_header
	number
	numeric_value
	permission
	permission_type
	predefined_type
	referenced_constant
	referenced_module
	referenced_type
	atext
	type

%type <bind_mode>
	binding

%type <transmit>
	transmission

%type <free_mode>
	allocation

%type <a_mode>
	address_mode

%type <d_mode>
	direction

%type <list>
	arg
	arg_list
	argument_list
	attribute_list
	candidate
	candidate_list
	connection
	connection_args
	connection_list
	connection_procedure
	correspondence
	correspondence_list
	designator
	designator_list
	error_list
	field
	field_list
	name_list
	permission_list
	referenced_module_list
	res
	res_list
	result_list


%start	module

%%

module :
		module_header module_interface module_body
		= {
			wrapup_module($1);
		}
	;

module_header :
		/*
		 * Allow simpler syntax.
		 */
		identifier '='
		= {
			module_header($1);
			$$ = $1;
		}
	|	identifier ':' PROGRAM prog_vers '='
		= {
			error(WARNING, "preferred syntax is \"<module name> = begin ... end.\"");
			module_header($1);
			$$ = $1;
		}
	;

prog_vers :
		/*
		 * We don't use the Courier program and version numbers.
		 */
		/* empty */
	|	number VERSION number
		= {
			error(WARNING, "program and version numbers are unnecessary");
		}
	;

module_interface :
		/* empty */
	|       interface_list
	;

interface_list :
		interface
	|       interface_list interface
	;

interface :
		CLIENT '=' BEGIN_ client_list END
	|       DIRECTORY '=' BEGIN_ directory_list END
	|       PROTOCOL '=' BEGIN_ protocol_list END
	|       SERVER '=' BEGIN_ server_list END
	|       STATISTICS '=' BEGIN_ statistics_list END
	;

client_list :
		/* empty */
	|       client_list client
	;

directory_list :
		/* empty */
	|       directory_list directory
	;

protocol_list :
		/* empty */
	|       protocol_list protocol
	;

server_list :
		/* empty */
	|       server_list server
	;

statistics_list :
		/* empty */
	|       statistics_list statistics
	;

client :
		BINDING '=' binding ';'
		= {
			client_bind( $3 );
		}
	|       REGION BEGIN_ '=' name_list ';'
		= {
			client_regionbegin( $4 );
		}
	|       REGION END '=' name_list ';'
		= {
			client_regionend( $4 );
		}
	|       CONNECTION '=' connection_procedure ';'
		= {
			client_connection( $3 );
		}
	|       PROTECTION '=' identifier ';'
		= {
			client_protection( $3 );
		}
	|       EXECUTION '=' identifier ';'
		= {
			client_execmode( $3 );
		}
	|       LOAD '=' identifier ';'
		= {
			client_load( $3 );
		}
	|       DIRECTORY '=' identifier ';'
		= {
			client_directory( $3 );
		}
	|       TIMEOUT ABSOLUTE '=' number ';'
		= {
			client_toabsolute( $4 );
		}
	|       TIMEOUT RELATIVE '=' number ';'
		= {
			client_torelative( $4 );
		}
	;

directory :
		ADDRESS '=' identifier number ';'
		= {
			directory_address( $3, $4 );
		}
	|       HOST '=' identifier ';'
		= {
			directory_host( $3 );
		}
	|       PORT '=' number ';'
		= {
			directory_port( $3 );
		}
	;

protocol :
		NAME '=' identifier ';'
		= {
			protocol_name( $3 );
		}
	|       DIRECTORY '=' identifier ';'
		= {
			protocol_directory( $3 );
		}
	|       LOAD '=' identifier ';'
		= {
			protocol_load( $3 );
		}
	;

server :
		REGION BEGIN_ '=' name_list ';'
		= {
			server_regionbegin( $4 );
		}
	|       REGION END '=' name_list ';'
		= {
			server_regionend( $4 );
		}
	|       CONNECTION '=' connection_procedure ';'
		= {
			server_connection( $3 );
		}
	|       ADDRESS '=' identifier number ';'
		= {
			server_address( $3, $4 );
		}
	|       HOST '=' identifier ';'
		= {
			server_host( $3 );
		}
	|       PORT '=' number ';'
		= {
			server_port( $3 );
		}
	|       EXECUTABLE FILE_ '=' atext ';'
		= {
			server_filename( $4 );
		}
	|       INTERFACE FILE_ '=' atext ';'
		= {
			server_interface( $4 );
		}
	|       PARAMETERS '=' atext ';'
		= {
			server_parameters( $3 );
		}
	|       PROTECTION '=' identifier ';'
		= {
			server_protection( $3 );
		}
	|       EXECUTION '=' identifier ';'
		= {
			server_execmode( $3 );
		}
	|       POLICY '=' identifier ';'
		= {
			server_policy( $3 );
		}
	|       MAXCONNECTIONS '=' number ';'
		= {
			server_maxconnections( $3 );
		}
	|       LOAD '=' identifier ';'
		= {
			server_load( $3 );
		}
	|       DIRECTORY '=' identifier ';'
		= {
			server_directory( $3 );
		}
	|       TIMEOUT ABSOLUTE '=' number ';'
		= {
			server_toabsolute( $4 );
		}
	|       TIMEOUT RELATIVE '=' number ';'
		= {
			server_torelative( $4 );
		}
	|       PERMISSION '=' permission_type ';'
		= {
			server_permission( $3 );
		}
	;

statistics :
		AUTHOR '=' author_list ';'
	|       VERSION '=' version_type ';'
	|       DATE '=' date_type ';'
	|       MODIFIED BY '=' author_list ';'
	|       MODIFIED DATE '=' date_type ';'
	;

author_list :
		author
	|       author_list ',' author
	;

author:
		identifier
	|       author period identifier
	;

version_type :
		identifier_or_number period identifier_or_number
	;

date_type :
		identifier_or_number period identifier_or_number period number
	;

identifier_or_number :
		identifier
	|       number
	;

binding :
		FILE_
		= {
			$$ = B_FILE;
		}
	|       FILE_ PROCEDURE
		= {
			$$ = B_FILE_PROCEDURE;
		}
	;

connection_procedure:
		identifier connection_args
		= {
			$$ = cons( $1, (list)procedure_type( (list)$2, NIL, NIL ) );
		}
	;

connection_args:
		/* empty */
		= {
			$$ = NIL;
		}
	|       '[' connection_list ']'
		= {
			$$ = $2;
		}
	;

connection_list:
		connection
		= {
			$$ = cons((list) $1, NIL);
		}
	|       connection_list ',' connection
		= {
			$$ = nconc($1, cons((list) $3, NIL));
		}
	;

connection:
		SERVICE
		= {
			$$ = make_connection( IE_SERVICE );
		}
	|       HOST
		= {
			$$ = make_connection( IE_HOSTNAME );
		}
	|       PORT
		= {
			$$ = make_connection( IE_PORTNUMBER );
		}
	|       PROTECTION
		= {
			$$ = make_connection( IE_PROTECTION );
		}
	|       EXECUTION
		= {
			$$ = make_connection( IE_EXECUTION );
		}
	|       LOAD
		= {
			$$ = make_connection( IE_LOAD );
		}
	|       FILE_
		= {
			$$ = make_connection( IE_FILENAME );
		}
	|       DIRECTORY
		= {
			$$ = make_connection( IE_DIRECTORY );
		}
	|       DIRECTORY HOST
		= {
			$$ = make_connection( IE_DIRHOST );
		}
	|       DIRECTORY PORT
		= {
			$$ = make_connection( IE_DIRPORT );
		}
	|       PROTOCOL
		= {
			$$ = make_connection( IE_PROTOCOL );
		}
	|       LOAD PROTOCOL
		= {
			$$ = make_connection( IE_LOADPROTOCOL );
		}
	|       DIRECTORY PROTOCOL
		= {
			$$ = make_connection( IE_DIRPROTOCOL );
		}
	|       TIMEOUT ABSOLUTE
		= {
			$$ = make_connection( IE_TOABSOLUTE );
		}
	|       TIMEOUT RELATIVE
		= {
			$$ = make_connection( IE_TORELATIVE );
		}
	;

permission_type:
		number
		= {
			$$ = $1;
		}
	|       permission
		= {
			$$ = $1;
		}
	;

module_body :
		BEGIN_ dependency_list declaration_list END period
		= {
			module_tail();
		}
	;

period :
		/* empty */
	|	'.'
	;

dependency_list :
		/* empty */
	|	DEPENDS UPON referenced_module_list ';'
		= {
			dependencies = nconc(dependencies, $3);
		}
	;

referenced_module_list :
		referenced_module
		= {
			$$ = cons((list) $1, NIL);
		}
	|	referenced_module_list ',' referenced_module
		= {
			$$ = nconc($1, cons((list) $3, NIL));
		}
	;

referenced_module :
		identifier
		= {
			ref_module($1);
			$$ = $1;
		}
	|	identifier '(' number ')' VERSION number
		= {
			error(WARNING, "program and version numbers are unnecessary");
			ref_module($1);
			$$ = $1;
		}
	;

declaration_list :
		/* empty */
	|	declaration_list declaration
	;

declaration :
		identifier ':' TYPE '=' type ';'
		= {
			define_type($1, $5);
		}
	|	identifier ':' type '=' numeric_value ';'
		= {
			if (basetype($3)->t_constr == C_PROCEDURE) {
				error(WARNING, "value for procedure %s ignored",
				     rep($1));
				define_procedure($1, $3);
			} else if (basetype($3)->t_constr == C_EXECUTION) {
				error(WARNING, "value for execution %s ignored",
				     rep($1));
				define_execution($1, $3);
			} else if (basetype($3)->t_constr == C_INTERRUPT) {
				error(WARNING, "value for interrupt %s ignored",
				     rep($1));
				define_interrupt($1, $3);
			} else if (basetype($3)->t_constr == C_MESSAGE) {
				error(WARNING, "value for message %s ignored",
				     rep($1));
				define_message($1, $3);
			} else if (basetype($3)->t_constr == C_FILE) {
				error(WARNING, "value for file %s ignored",
				     rep($1));
				define_file($1, $3);
			} else if (basetype($3)->t_constr == C_MEMORY) {
				error(WARNING, "value for memory %s ignored",
				     rep($1));
				define_memory($1, $3);
			} else if (type_check($3, $5)) {
				define_constant($1, $3, $5);
			} else {
				error(SERIOUS, "type clash in definition of %s",
					rep($1));
			}
		}
	|	identifier ':' type ';'
		= {
			if (basetype($3)->t_constr == C_PROCEDURE) {
				define_procedure($1, $3);
			} else if (basetype($3)->t_constr == C_EXECUTION) {
				define_execution($1, $3);
			} else if (basetype($3)->t_constr == C_INTERRUPT) {
				define_interrupt($1, $3);
			} else if (basetype($3)->t_constr == C_MESSAGE) {
				define_message($1, $3);
			} else if (basetype($3)->t_constr == C_FILE) {
				define_file($1, $3);
			} else if (basetype($3)->t_constr == C_MEMORY) {
				define_memory($1, $3);
			} else if (basetype($3)->t_constr == C_GLOBAL) {
				define_global($1, $3);
			} else {
				error(SERIOUS, "no value specified for %s", rep($1));
			}
		}
	;

type :
		predefined_type
		= {
			$$ = $1;
		}
	|	constructed_type
		= {
			$$ = $1;
		}
	|	referenced_type
		= {
			$$ = $1;
		}
	;

predefined_type :
		BOOLEAN
		= {
			$$ = boolean_type;
		}
	|       BYTE
		= {
			$$ = byte_type;
		}
	|       CHAR
		= {
			$$ = character_type;
		}
	|       SMALL
		= {
			$$ = small_type;
		}
	|       SHORT
		= {
			$$ = integer_type;
		}
	|	INTEGER
		= {
			$$ = integer_type;
		}
	|       SHORT INTEGER
		= {
			$$ = integer_type;
		}
	|       LONG
		= {
			$$ = long_integer_type;
		}
	|	LONG INTEGER
		= {
			$$ = long_integer_type;
		}
	|       HYPER
		= {
			$$ = hyper_type;
		}
	|       UNSIGNED SMALL
		= {
			$$ = usmall_type;
		}
	|       UNSIGNED SHORT
		= {
			$$ = cardinal_type;
		}
	|       CARDINAL
		= {
			$$ = cardinal_type;
		}
	|       SHORT CARDINAL
		= {
			$$ = cardinal_type;
		}
	|       UNSIGNED LONG
		= {
			$$ = long_cardinal_type;
		}
	|	LONG CARDINAL
		= {
			$$ = long_cardinal_type;
		}
	|       UNSIGNED HYPER
		= {
			$$ = uhyper_type;
		}
	|       FLOAT
		= {
			$$ = floating_type;
		}
	|       DOUBLE
		= {
			$$ = long_floating_type;
		}
	|       LONG FLOAT
		= {
			$$ = long_floating_type;
		}
	|	STRING
		= {
			$$ = string_type;
		}
	|       STRING_REF
		= {
			$$ = string_ref_type;
		}
	|	UNSPECIFIED
		= {
			$$ = unspecified_type;
		}
	|       SHORT UNSPECIFIED
		= {
			$$ = unspecified_type;
		}
	|	LONG UNSPECIFIED
		= {
			$$ = long_unspecified_type;
		}
	;

constructed_type :
		'{' correspondence_list '}'
		= {
			$$ = enumeration_type($2);
		}
	|       CONSTANT type '{' correspondence_list '}'
		= {
			$$ = constant_type($2, $4);
		}
	|	ARRAY numeric_value OF type
		= {
			$$ = array_type($2, $4);
		}
	|	SEQUENCE maximum_number OF type
		= {
			$$ = sequence_type($2, $4, 1);
		}
	|       SEQUENCE_REF maximum_number OF type
		= {
			$$ = sequence_type($2, $4, 0);
		}
	|       NC_SEQUENCE maximum_number OF type
		= {
			$$ = nc_sequence_type($2, $4, 1);
		}
	|       NC_SEQUENCE_REF maximum_number OF type
		= {
			$$ = nc_sequence_type($2, $4, 0);
		}
	|       RECORD attribute_list
		= {
			$$ = record_type($2);
		}
	|	CHOICE designator_type OF '{' candidate_list '}'
		= {
			$$ = choice_type($2, $5);
		}
	|       PROCEDURE argument_list result_list error_list permission_list
		= {
			$$ = procedure_type($2, $3, $4, $5);
		}
	|       EXECUTION argument_list result_list error_list permission_list
		= {
			$$ = execution_type($2, $3, $4, $5);
		}
	|       INTERRUPT argument_list result_list error_list permission_list
		= {
			$$ = interrupt_type($2, $3, $4, $5);
		}
	|       MESSAGE argument_list result_list error_list permission_list
		= {
			$$ = message_type($2, $3, $4, $5);
		}
	|       FILE_ argument_list result_list error_list permission_list
		= {
			$$ = file_type($2, $3, $4, $5);
		}
	|       MEMORY argument_list result_list error_list permission_list
		= {
			$$ = memory_type($2, $3, $4, $5);
		}
	|       ERROR attribute_list
		= {
			$$ = error_type($2);
		}
	|       GLOBAL argument_list
		= {
			$$ = global_type($2);
		}
	|       PRIVATE
		= {
			$$ = private_type();
		}
	;

referenced_type :
		identifier
		= {
			if (check_def($1))
				$$ = $1;
			else
				$$ = unspecified_type;
		}
	|	identifier '.' identifier
		= {
			if (check_dependency($1))
				$$ = $3;
			else
				$$ = unspecified_type;
		}
	;

correspondence_list :
		correspondence
		= {
			$$ = cons($1, NIL);
		}
	|	correspondence_list ',' correspondence
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

correspondence :
		identifier '(' numeric_value ')'
		= {
			$$ = cons((list) $1, (list) $3);
		}
	;

maximum_number :
		numeric_value
		= {
			$$ = $1;
		}
	|	/* empty */
		= {
			$$ = ONIL;
		}
	;

numeric_value :
		number
		= {
			$$ = $1;
		}
	|	referenced_constant
		= {
			$$ = $1;
		}
	;

designator_type :
		/* empty */
		= {
			$$ = ONIL;
		}
	|       type
		= {
			$$ = $1;
		}
	;

candidate_list :
		candidate
		= {
			$$ = cons($1, NIL);
		}
	|	candidate_list ',' candidate
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

candidate :
		/*
		 * Yes, this production lets the user get away with putting
		 * white space between the '=' and the '>'.
		 */
		designator_list '=''>' type
		= {
			$$ = cons($1, (list) $4);
		}
	;

designator_list :
		designator
		= {
			$$ = cons($1, NIL);
		}
	|	designator_list ',' designator
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

designator :
		identifier
		= {
			$$ = cons((list) $1, NIL);
		}
	|	correspondence
		= {
			$$ = $1;
		}
	;

attribute_list :
		/* empty */
		= {
			$$ = NIL;
		}
	|       '[' field_list ']'
		= {
			$$ = $2;
		}
	;

argument_list :
		/* empty */
		= {
			$$ = NIL;
		}
	|       '[' arg_list ']'
		= {
			$$ = $2;
		}
	;

result_list :
		/* empty */
		= {
			$$ = NIL;
		}
	|       RETURNS '[' res_list ']'
		= {
			$$ = $3;
		}
	;

error_list :
		/* empty */
		= {
			$$ = NIL;
		}
	|       EXCEPTION '[' name_list ']'
		= {
			$$ = $3;
		}
	;

permission_list :
		/* empty */
		= {
			$$ = cons( (list)make_constant( NP_F_POREAD ), NIL );
		}
	|       PERMISSION '[' access_type ']'
		= {
			$$ = cons( (list)$3, NIL );
		}
	;

field_list :
		field
		= {
			$$ = cons($1, NIL);
		}
	|	field_list ',' field
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

field :
		name_list ':' type
		= {
			$$ = cons($1, (list) $3);
		}
	;

arg_list :
		arg
		= {
			$$ = cons($1, NIL);
		}
	|       arg_list ',' arg
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

arg :
		name_list ':' type transmission
		= {
			$$ = cons((list) make_arg($1, $4), (list) type_conv($3));
		}
	;

res_list :
		res
		= {
			$$ = cons($1, NIL);
		}
	|       res_list ',' res
		= {
			$$ = nconc($1, cons($3, NIL));
		}
	;

res :
		name_list ':' type transmission
		= {
			$$ = cons((list) make_arg($1, $4), (list) type_conv($3));
		}
	;

transmission:
		address_mode direction allocation
		= {
			$$ = make_transmission($1, $2, $3);
		}
	;

allocation :
		/* empty */
		= {
			$$ = F_USER;
		}
	|       AUTOFREE
		= {
			$$ = F_AUTOFREE;
		}
	;

address_mode :
		/* empty */
		= {
			$$ = A_VALUE;
		}
	|       VALUE
		= {
			$$ = A_VALUE;
		}
	|       BY VALUE
		= {
			$$ = A_VALUE;
		}
	|       REF
		= {
			$$ = A_REF;
		}
	|       BY REF
		= {
			$$ = A_REF;
		}
	;

direction :
		/* empty */
		= {
			$$ = D_IN;
		}
	|       IN
		= {
			$$ = D_IN;
		}
	|       OUT
		= {
			$$ = D_OUT;
		}
	|       IN OUT
		= {
			$$ = D_INOUT;
		}
	|       INOUT
		= {
			$$ = D_INOUT;
		}
	;

access_type :
		READ
		= {
			$$ = make_constant( NP_F_POREAD );
		}
	|       WRITE
		= {
			$$ = make_constant( NP_F_POWRITE );
		}
	|       READ WRITE
		= {
			$$ = make_constant( NP_F_POREAD | NP_F_POWRITE );
		}
	;

referenced_constant :
		identifier
		= {
			if (check_def($1))
				$$ = $1;
			else
				$$ = undefined_constant;
		}
	|	identifier '.' identifier
		= {
			if (check_dependency($1))
				$$ = $3;
			else
				$$ = undefined_constant;
		}
	;

name_list :
		identifier
		= {
			$$ = cons((list) $1, NIL);
		}
	|	name_list ',' identifier
		= {
			$$ = nconc($1, cons((list) $3, NIL));
		}
	;

%%

#if defined(SUNSPARC)
extern YYSTYPE *yyv;
#endif

extern int yynerrs;
extern int yylineno;

#if defined(HPPA)
extern YYSTYPE yyv[];
extern int yyerrflag;
#endif

#if defined(LINUX386) || defined(LINUXAMD64)
YYSTYPE yyv[YYMAXDEPTH];
#endif


struct parser_state {
	YYSTYPE yyv[YYMAXDEPTH];
	YYSTYPE yylval;
	YYSTYPE yyval;
	int yychar;
	int yynerrs;
	short yyerrflag;
	int yylineno;
	int recursive_flag;
};

save_parser_state()
{
	struct parser_state *p;

	p = new(struct parser_state);
	memcpy(p->yyv, yyv, YYMAXDEPTH*sizeof(YYSTYPE));
	p->yylval = yylval;
	p->yyval = yyval;
	p->yychar = yychar;
	p->yynerrs = yynerrs;
	p->yyerrflag = yyerrflag;
	p->yylineno = yylineno;
	p->recursive_flag = recursive_flag;
	recursive_flag = 1;
	return ((int) p);
}

restore_parser_state(p)
	struct parser_state *p;
{
	recursive_flag = p->recursive_flag;
	yylineno = p->yylineno;
	yyerrflag = p->yyerrflag;
	yynerrs = p->yynerrs;
	yychar = p->yychar;
	yyval = p->yyval;
	yylval = p->yylval;
	memcpy(yyv, p->yyv, YYMAXDEPTH*sizeof(YYSTYPE));
	free((char *) p);
}
