/*
 * Copyright (C) 1986 by Inge B. Kristiansen
 * All rights reserved.
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Make a unique identifier for a service name. The identifier is used
 * on the server side when more than one service is exported.
 */

unsigned short
hash_service( service_name )
	char *service_name;
{
	unsigned short i;
	unsigned short ident = 0;

	/* Just make a position weighted sum of the characters in the   */
	/* service name.                                                */
	for( i = 1; *service_name; ++i, ++service_name ) {
		ident += (((unsigned short)*service_name) * i);
	}
	return( ident );
}


/*
 * Make a unique checksum for all the types contained in a procedure
 * call, execution etc. This checksum is used to determine if the
 * client and server side of a service are consistent.
 */

unsigned long
hash_type( type )
	struct object *type;
{
	list p, q;
	unsigned int nresults;
	unsigned long ident = 0;
	unsigned long i, typeid, addrid, dirid;
	struct object *t, *bt;

	/* Loop through all the arguments in the argument list.         */
	for (p = type->t_args, i = 1; p != NIL; p = cdr(p), ++i) {
		q = car(car(p));
		t = ocdr(car(p));
		bt = basetype(t);
		typeid = ((unsigned long)bt->t_id) + 1L;
		ident += typeid;
		addrid = (unsigned long)ocar(q)->o_addr;
		ident += addrid;
		dirid  = (unsigned long)ocar(q)->o_dir;
		ident += dirid;
		ident = ident * i;
	}

	/* Include the results in the checksum as well.                 */
	nresults = length(type->t_results);
	if (nresults) {
		p = type->t_results;
		q = car(car(p));
		t = ocdr(car(p));
		bt = basetype(t);
		typeid = ((unsigned long)bt->t_id) + 1L;
		ident += typeid;
	}
	return( ident );
}
