/*
 * Copyright (C) 1984 by Inge B. Kristiansen.
 * All rights reserved.
 */

#include "sysconfig.h"
#include "np.h"
#if !defined(NO_STDLIB_H)
#include <stdlib.h>
#else
extern char *malloc(), *calloc();
#endif
#include <string.h>

/*
 * String allocation.
 */
char *
copy(s)
	char *s;
{
	char *p;

	if ((p = malloc((unsigned) strlen(s)+1)) == NULL)
		error(FATAL, "out of string space");
	(void) strcpy(p, s);
	return (p);
}

/*
 * Object allocation.
 */
struct object *
make_type(constr)
	enum constr constr;
{
	struct object *o;

	o = new(struct object);
	o->o_class = O_TYPE;
	o->o_cmod = O_NORMAL;
	o->o_type = new(struct type);
	o->t_constr = constr;
	o->t_id = T_COMPOSITE;
	return (o);
}

struct object *
make_symbol(name)
	char *name;
{
	struct object *o;

	o = new(struct object);
	o->o_class = O_SYMBOL;
	o->o_cmod = O_NORMAL;
	o->o_name = copy(name);
	return (o);
}

struct object *
make_constant(value)
	int value;
{
	struct object *o;

	o = new(struct object);
	o->o_class = O_CONSTANT;
	o->o_cmod = O_NORMAL;
	o->o_value = value;
	return (o);
}

struct object *
make_text(string)
	char *string;
{
	struct object *o;

	o = new(struct object);
	o->o_class = O_TEXT;
	o->o_cmod = O_NORMAL;
	o->o_text = copy(string);
	return (o);
}

list
make_arg(l, t)
	list l;
	struct transmit *t;
{
	struct object *o;

	o = ocar(l);
	if( o->o_class != O_SYMBOL && o->o_class != O_CONSTANT )
		error(FATAL, "illegal object class in make_arg");
	o->o_cmod = O_ARG;
	o->o_transmit = t;
	return (l);
}

struct transmit *
make_transmission(address, direction, freealloc)
	enum addr address;
	enum dir direction;
	enum freeing freealloc;
{
	struct transmit *t;

	t = new(struct transmit);
	t->t_freeing = freealloc;
	t->t_addr = address;
	t->t_dir = direction;
/* This would be too simplified - globals and arrays do not need ref !!!
	if (t->t_dir != D_IN && t->t_addr != A_REF)
		error(WARNING, "in C return parameters should be by reference");
*/
	if (t->t_freeing != F_USER && (t->t_dir == D_IN || t->t_addr != A_REF) )
		error(WARNING, "free/allocation attribute can only be used with ref (in)out parameters");
	return (t);
}

char *
make_prefix(name)
	char *name;
{
	char *delimiter;
	char *p;

	/* Allocate space for name plus the delimiter.                  */
	delimiter = NP_PREFIX_DELIMITER;
	if ((p = malloc((unsigned) strlen(name)+strlen(delimiter)+1)) == NULL)
		error(FATAL, "out of string space");
	(void) strcpy(p, name);
	(void) strcat(p, delimiter);
	return (p);
}

/*
 * Return a printable representation of an object (number or string).
 */
char *
rep(o)
	struct object *o;
{
	char buf[MAXSTR];

	switch (o->o_class) {
	    case O_CONSTANT:
		(void) sprintf(buf, "%d", o->o_value);
		return (copy(buf));
	    case O_SYMBOL:
		return (o->o_name);
	    case O_TEXT:
		return (o->o_text);
	    case O_TYPE:
		return (o->t_name);
	    default:
		error(FATAL, "internal error (rep): bad class");
		/* NOTREACHED */
	}
}

/*
 * Return a printable representation of an object (number or string).
 * and free the object itself.
 */
char *
repfree(o)
	struct object *o;
{
	char *ovalue;

	ovalue = rep(o);
	free(o);
	return(ovalue);
}

/*
 * Return the constant value of an object (only number).
 */
int
recons(o)
	struct object *o;
{
	switch (o->o_class) {
	    case O_CONSTANT:
		return (o->o_value);
	    default:
		error(FATAL, "internal error (recons): bad class");
		/* NOTREACHED */
	}
}

/*
 * Return the constant value of an object (only number),
 * and free the object itself.
 */
int
reconsfree(o)
	struct object *o;
{
	int ovalue;

	ovalue = recons(o);
	free(o);
	return (ovalue);
}

/*
 * Lisp operations.
 */
list
cons(a, b)
	list a, b;
{
	list p;

	if ((p = new(struct cons)) == NIL)
		error(FATAL, "out of list space");
	car(p) = a;
	cdr(p) = b;
	return (p);
}

length(p)
	list p;
{
	int n;

	for (n = 0; p != NIL; p = cdr(p), n += 1)
		;
	return (n);
}

list
nconc(p, q)
	list p, q;
{
	list pp;

	pp = p;
	if (p == NIL)
		return (q);
	while (cdr(p) != NIL)
		p = cdr(p);
	cdr(p) = q;
	return (pp);
}

member(symbol, p)
	struct object *symbol;
	list p;
{
	char *name;

	name = symbol->o_name;
	while (p != NIL)
		if (streq(ocar(p)->o_name, name))
			return (1);
		else
			p = cdr(p);
	return (0);
}
