/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Global data structures maintained by this module.
 */

struct server   c_server = {            /* Server control block */
		NP_SREGIONBEGIN,        /* Default region begin */
		NP_SREGIONEND,          /* Default region end */
		NP_SCONNECTION,         /* Default export mode */
		NP_SHOSTNAME,           /* Default host name */
		NP_SPORTNUMBER,         /* Default port number */
		NP_SEXECFILE,           /* Default executable file name */
		NP_SIDFFILE,            /* Default interface description file name */
		NP_SPARAMETERS,         /* Default parameters */
		NP_SPROTECTION,         /* Default protection */
		NP_SEXECUTION,          /* Default execution mode */
		NP_SPOLICY,             /* Default scheduling policy */
		NP_SMAXCONN,            /* Default max connections */
		NP_SLOAD,               /* Default load mode */
		NP_SDIRECTORY,          /* Default directory */
		NP_STOABSOLUTE,         /* Default connection lifetime */
		NP_STORELATIVE,         /* Default idle time for connection */
		NP_SPERMISSION          /* Default server permission flags */
};

static char     *c_string = "server";   /* Used for error reporting */


/*
 * Set the server region BEGIN information.
 */

server_regionbegin( region )
	list region;
{
	check_attribute( c_string );
	c_server.s_regionbegin = region;
}


/*
 * Set the server region END information.
 */

server_regionend( region )
	list region;
{
	check_attribute( c_string );
	c_server.s_regionend = region;
}

/*
 * Initialize the server connection information. This procedure is
 * called before the parsing starts.
 */

server_initconnection()
{
	list arglist;
	list exportlist;
	list connlist;

	arglist = cons( (list)make_connection( IE_PORTNUMBER ), NIL );
	exportlist = cons( make_symbol("export"), (list)procedure_type( (list)arglist, NIL, NIL ) );
	connlist = cons( cons( exportlist, connections ), NIL );
	c_server.s_connection = connlist;
	connections = NIL;
}

/*
 * Set the server connection information.
 */

server_connection( exportlist )
	list exportlist;
{
	list connlist;
	static short aflag = 0;

	if( !aflag ) {
		++aflag;
		connlist = cons( cons( exportlist, connections ), NIL );
	} else {
		connlist = c_server.s_connection;
		connlist = nconc( connlist, cons( cons( exportlist, connections ), NIL ) );
	}
	c_server.s_connection = connlist;
	connections = NIL;
}

/*
 * Set the host name.
 */

server_host( name )
	struct object *name;
{
	check_attribute( c_string );
	c_server.s_hostname = repfree( name );
}

/*
 * Set the port number.
 */

server_port( number )
	struct object *number;
{
	check_attribute( c_string );
	c_server.s_portnumber = reconsfree( number );
}

/*
 * Set the complete address.
 */

server_address( name, number )
	struct object *name;
	struct object *number;
{
	server_host( name );
	server_port( number );
}

/*
 * Set the executable file name.
 */

server_filename( name )
	struct object *name;
{
	check_attribute( c_string );
	c_server.s_execfile = repfree( name );
}

/*
 * Set the interface description file name.
 */

server_interface( name )
	struct object *name;
{
	check_attribute( c_string );
	c_server.s_idffile = repfree( name );
}

/*
 * Set the parameters.
 */

server_parameters( string )
	struct object *string;
{
	check_attribute( c_string );
	c_server.s_parameters = repfree( string );
}

/*
 * Set the server protection information.
 */

server_protection( protection )
	struct object *protection;
{
	check_attribute( c_string );
	c_server.s_protection = repfree( protection );
}

/*
 * Set the server execution information.
 */

server_execmode( execution )
	struct object *execution;
{
	check_attribute( c_string );
	c_server.s_execution = repfree( execution );
}

/*
 * Set the server policy information.
 */

server_policy( policy )
	struct object *policy;
{
	check_attribute( c_string );
	c_server.s_policy = repfree( policy );
}

/*
 * Set the server execution information.
 */

server_maxconnections( number )
	struct object *number;
{
	check_attribute( c_string );
	c_server.s_maxconn = reconsfree( number );
}

/*
 * Set the server load information.
 */

server_load( load )
	struct object *load;
{
	check_attribute( c_string );
	c_server.s_load = repfree( load );
}

/*
 * Set the server directory information.
 */

server_directory( directory )
	struct object *directory;
{
	check_attribute( c_string );
	c_server.s_directory = repfree( directory );
}

/*
 * Set the connection life time.
 */

server_toabsolute( number )
	struct object *number;
{
	check_attribute( c_string );
	c_server.s_toabsolute = reconsfree( number );
}

/*
 * Set the no response timeout.
 */

server_torelative( number )
	struct object *number;
{
	check_attribute( c_string );
	c_server.s_torelative = reconsfree( number );
}

/*
 * Set the server permission flags.
 */

server_permission( number )
	struct object *number;
{
	check_attribute( c_string );
	c_server.s_permission = reconsfree( number );
}
