/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 */

#include "sysconfig.h"
#include "np.h"


/*
 * Symbol table management.
 */
struct object *
lookup(symlist, symbol)
	list symlist;
	struct object *symbol;
{
	char *name;
	list p, q;

	if (symbol->o_class != O_SYMBOL)
		error(FATAL, "internal error (lookup): not a symbol");
	name = symbol->o_name;
	for (p = symlist; p != NIL; p = cdr(p)) {
		q = car(p);
		if (streq(ocar(q)->o_name, name))
			return (ocdr(q));
	}
	return (0);
}

check_def(symbol)
	struct object *symbol;
{
	if (lookup(values, symbol) == 0) {
		error(SERIOUS, "%s undefined", symbol->o_name);
		return (0);
	}
	return (1);
}

declare(symlist, name, value)
	list *symlist;
	struct object *name, *value;
{
	if (lookup(*symlist, name) != 0) {
		error(SERIOUS, "%s redeclared", name->o_name);
		return;
	}
	*symlist = nconc(*symlist, cons(cons((list) name, (list) value), NIL));
}

/*
 * Find the underlying value of a constant.
 */
true_value(constant)
	struct object *constant;
{
	if (constant == ONIL) /* possible in case of sequences */
		return (0);
	while (constant != 0 && constant->o_class == O_SYMBOL)
		constant = lookup(values, constant);
	if (constant == 0 || constant->o_class != O_CONSTANT)
		error(FATAL, "internal error (true_value): not a constant");
	return (constant->o_value);
}

char *
gensym()
{
	char buf[MAXSTR];
	static int n = 0;

	(void) sprintf(buf, "_type%d_%s", ++n, module_name);
	return (copy(buf));
}

#ifdef DEBUG

/*
 * Print out all declarations.
 */
print_declarations(f, symtab)
	FILE *f;
	list symtab;
{
	list p;

	for (p = symtab; p != NIL; p = cdr(p)) {
		if (ocar(car(p)) != undefined_constant)
			print_definition(f, ocar(car(p)), ocdr(car(p)), symtab);
	}
}

print_definition(f, symbol, value, symtab)
	FILE *f;
	struct object *symbol;
	struct object *value;
	list symtab;
{
	char *name;
	struct object *type;

	name = symbol->o_name;
	switch (value->o_class) {
	    case O_TYPE:
		fprintf(f, "type %s ", name);
		print_type(f, value);
		fprintf(f, "\n");
		break;
	    case O_CONSTANT:
		type = lookup(symtab, symbol);
		fprintf(f, "constant %s ", name);
		print_type(f, type);
		fprintf(f, " %d\n", value->o_value);
		break;
	    case O_SYMBOL:
		fprintf(f, "symbol %s %s\n", name, value->o_name);
		if (value->o_name != 0)
			print_declarations(f, value->o_name);
		break;
	}
}

print_type(f, type)
	FILE *f;
	struct object *type;
{
	list p, q;
	struct object *t;
	char *member;

	if (type->o_class == O_SYMBOL) {
		fprintf(f, "%s", type->o_name);
		return;
	}
	switch (type->t_constr) {
	    case C_PREDEF:
		fprintf(f, "%s", type->t_name);
		break;
	    case C_ENUMERATION:
		fprintf(f, "(enumeration ");
		for (p = type->t_list; p != NIL; p = cdr(p)) {
			q = car(p);
			fprintf(header, " (%s %s)", ocar(q)->o_name, rep(ocdr(q)));
		}
		fprintf(f, ")");
		break;
	    case C_ARRAY:
		fprintf(f, "(array %s ", rep(type->t_size));
		print_type(f, type->t_basetype);
		fprintf(f, ")");
		break;
	    case C_SEQUENCE:
		fprintf(f, "(sequence ");
		print_type(f, type->t_basetype);
		fprintf(f, ")");
		break;
	    case C_RECORD:
		fprintf(f, "(record");
		for (p = type->t_list; p != NIL; p = cdr(p)) {
			t = ocdr(car(p));
			for (q = car(car(p)); q != NIL; q = cdr(q))
				fprintf(f, " (%s %s)", ocar(q)->o_name, rep(t)); 
		}
		fprintf(f, ")");
		break;
	    case C_CHOICE:
		fprintf(f, "(choice");
		for (p = type->t_candidates; p != NIL; p = cdr(p)) {
			for (q = car(car(p)); q != NIL; q = cdr(q)) {
				t = ocar(car(q));
				fprintf(f, " (%s %s ", t->o_name,
					rep(enum_value(t, type->t_designator)));
				print_type(f, ocdr(car(p)));
				fprintf(f, ")");
			}
		}
		fprintf(f, ")");
		break;
	    default:
		error(FATAL, "internal error (print_type): bad type constructor");
	}
}

#endif
