/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 *
 * Modifications:
 *	16-03-90 (turch) p_socket made recallabe if it is allowed
 *			 NC_PRX_PUBLIC... replaces RUDP definitions
 */

#include "sysconfig.h"
#include "ncc.h"
#include "nc.h"


/*
 * Global variable controlling the initiation of the protocol.
 */

int nc_pinitiated = 0;


/*
 * Initiate the protocol communication endpoint. That is initialize
 * the socket over which the communication will go.
 */

int
nc_pinit(struct protocol *p, struct nct_common_service *cd)
{
	timeouts *t;
	extern timeouts *nc_tmformat(struct nct_common_service *cd);
	int status;

	/* Ensure that the initialization is only performed once for    */
	/* the communication endpoint if multiple call not allowed      */
	if( !nc_pinitiated || (p->p_flags & NC_P_FINIT)) 
		{
		/* Construct the timeout structure to be used.  */
		t = nc_tmformat( cd );

		/* Create the communication endpoint.		*/
		if( (status = (*p->p_socket)( cd->nc_localport,
				nc_setprotection( cd ), cd->nc_execution, t )) < 0 ) {
			return( status );
		} else
			cd->nc_localport = status;
		nc_pinitiated = 1;
	}
	return( status );
}


/*
 * Construct the protection mode to be used for the lower level
 * transport software. TO BE CHANGED.
 */

int
nc_setprotection(struct nct_common_service *cd)
{
	if( cd->nc_load == NC_L_PERMANENT ) {
		if( cd->nc_protection == NC_PR_PUBLIC ) {
			return( NC_PRX_PUBLIC );
		} else {
			return( NC_PRX_PRIVATE );
		}
	} else if( cd->nc_load == NC_L_TEMPORARY ) {
		if( cd->nc_protection == NC_PR_PUBLIC ) {
			return( NC_PRX_TPUBLIC );
		} else {
			return( NC_PRX_TPRIVATE );
		}
	} else {
		if( cd->nc_protection == NC_PR_PUBLIC ) {
			return( NC_PRX_PUBLIC );
		} else {
			return( NC_PRX_PRIVATE );
		}
	}
}
