/*
 * Copyright (C) 1986 by Inge B. Kristiansen.
 * All rights reserved.
 */

#include "sysconfig.h"
#if !defined(NO_STDLIB_H)
#include <stdlib.h>
#endif
#ifdef BSDstrlib
#include <strings.h>
#else
#include <string.h>
#endif
#include "ncc.h"
#include "nc.h"


/*
 * Compute the size of a string in bytes. The trailing zero is
 * also counted.
 */

sizeof_string(string *p)
{
	if( p && *p ) {
		return( strlen( *p ) + sizeof(idl_ushort_int) + 1 );
	} else {
		return( sizeof(idl_ushort_int) );
	}
}


/*
 * Externalize a string to the network. The trailing zero is
 * also transmitted.
 */

externalize_string(string *p, register _btype *buf)
{
	register string s;
	register int n;
	cardinal nn;

	/* Check if the pointer is zero.                                */
	if( p && ( s = *p ) ) {
	      n = nn = strlen( s ) + 1;
	} else {
	      s = "";
	      n = nn = 0;
	}

	/* Move the string out in the buffer.                           */
	buf += externalize_idl_ushort_int( &nn, buf );
	strcpy( buf, s );

	/* Return increment of the buffer pointer.                      */
	return( n + sizeof(idl_ushort_int) );
}


/*
 * Internalize a string from the network. There is a trailing zero
 * after the string.
 */

internalize_string(string *p, register _btype *buf)
{
	register string s;
	register int n;
	cardinal nn;

	/* Get the length of the string.                                */
	buf += internalize_idl_ushort_int( &nn, buf );
	n = nn;

	/* Check if the string is empty or not.                         */
	if( n ) {
		s = nc_allocate( n );
		strncpy( s, buf, n );
		s[n-1] = '\0';
	} else {
		s = (char *)0;
	}

	/* Return the string and the increment for the buffer pointer.  */
	*p = s;
	return( n + sizeof(idl_ushort_int) );
}

/*
 * Internalize a string without allocating memory. There is a trailing zero
 * after the string.
 */

internalize_string_ref(string *p, register _btype *buf)
{
	register string s;
	register int n;
	cardinal nn;

	/* Get the length of the string.                                */
	buf += internalize_idl_ushort_int( &nn, buf );
	n = nn;

	/* Check if the string is empty or not.                         */
	if( n ) {
		s = *p;
		strncpy( s, buf, n );
		s[n-1] = '\0';
	} else {
		s = (char *)0;
	}

	/* Return increment for the buffer pointer.  */
	return( n + sizeof(idl_ushort_int) );
}
