/* %W% %E% */
/*
 * nc.h - internal definitions for the nc rpc
 *
 * 07.05.90 kk  added call descriptor in nct_common_service (for rx)
 *
 * 13.10.92 kk  merged with nc_c.h
 * CS - 07.09.2000 : removed an unsed conditional include
 */


#include <stdio.h>      /* for TRACE */

#include "setjmp.h"

#include "queue.h"
#if !defined(_NO_TIMEVAL_DEF_)
#include <sys/time.h>
#endif
#include "timer.h"
#include "rx.h"

#define NC_FREE         0
#define NC_EXPORTED     1
#define NC_IMPORTED     2
#define NC_NODAL        3
#define NC_RESERVED     4



/*
 * Some common constants for Nodal and C.
 */
#define NC_MAXSERVICES  100             /* Max number of services in a process */

#define NCC_MAXSTR      80              /* Maximum string length */
#define NCC_NOSERVICE   -1              /* No service in rreceive() */
#define NCC_NOPORT      0               /* No port in rreceive() */
#define NCC_NORANDOMID  0L              /* No random id in rreceive() */
#define NCC_NOCONNECT   -1              /* No connection established yet */
#define NCC_MAXSYSERR   100             /* Max range for OS system errors */
#define NCC_POLICY      NC_SC_RROBIN    /* Scheduling policy for message receive */

/*
 * Nodal protocol definitions.
 */

#define NCN_MAXRECV     077777          /* Max size of recived data */
#define NCN_NODAL_COMMAND       "$DO\"ERASE %d.0%d; LOAD %s%s; RUN\""
#define NCN_NOARGBYTES          8
#define NCN_GROUP               1
#define NCN_STEP                1
#define NCN_STATUSBYTES         1
#define NCN_LOCALCREAT          0644

/*
 * Host and file name addressing by number.
 */

#define NCN_HOSTNSIZE           10
#define NCN_HOSTNUMBER          "$%d"
#define NCN_FILENSIZE           30
#define NCN_FILE1               "(%d)<%d>%s"
#define NCN_FILE2               "(%d)%s"
#define NCN_FILE3               "<%d>%s"


/*
 * Remote procedure call protocol header.
 */
typedef struct ncct_call_header {
	unsigned char   ncc_byteorder;                  /* 0 */
	unsigned char   ncc_floattype;
	unsigned char   ncc_version;
	unsigned char   ncc_revision;
	nc_32uint       ncc_length;                     /* 4 */
	nc_32uint       ncc_random_id;                  /* 8 */
	unsigned short  ncc_service_id;                 /* 12 */
	unsigned short  ncc_procedure_number;
	unsigned short  ncc_userid;                     /* 16 */
	unsigned short  ncc_groupid;
	unsigned short  ncc_type;                       /* 20 */
	unsigned short  ncc_errno;
	unsigned char   ncc_reserved[4];                /* 24 */
							/* 28 */
} ncct_call_header;

/*
 * Remote procedure return protocol header.
 */

typedef struct ncct_return_header {
	unsigned char   ncc_byteorder;                  /* 0 */
	unsigned char   ncc_floattype;
	unsigned char   ncc_version;
	unsigned char   ncc_revision;
	nc_32uint       ncc_length;                     /* 4 */
	unsigned short  ncc_errtype;                    /* 8 */
	unsigned short  ncc_errcode;
	unsigned short  ncc_type;                       /* 12 */
	unsigned char   ncc_reserved[2];                /* 14 */
							/* 16 */
} ncct_return_header;


/*
 * Service entry table layout.
 */

struct  nct_cservice;                   /* Predeclare C services */
struct  nct_nservice;                   /* Predeclare Nodal services */

typedef struct nct_service {
	int     nc_servicetype;
	union {
		struct  nct_cservice    *nc_cconnection;
		struct  nct_nservice    *nc_nconnection;
	} nc_uservice;
} nct_service;
#define nc_cservice     nc_uservice.nc_cconnection
#define nc_nservice     nc_uservice.nc_nconnection


typedef enum nc_ct {
	CT_LOCAL,
	CT_REMOTE
} nc_ct;

/*
 * Common part of service entry for Nodal and C.
 */

typedef struct nct_common_service {
struct nct_common_service *nc_link;
char            *nc_name;
unsigned short  nc_flags;
short           nc_uid;
short           nc_gid;
struct timeval  nc_time;
nc_address      nc_addr;
int             nc_protection;
int             nc_execution;
int             nc_load;
char            *nc_filename;
int             nc_directory;
nc_address      nc_diraddr;
protocol        *nc_protocol;
protocol        *nc_loadprotocol;
protocol        *nc_dirprotocol;
long            nc_connlife;
long            nc_noresponse;
char            *nc_idffile;
char            *nc_parameters;
int             nc_policy;
int             nc_maxconn;
unsigned short  nc_service;
unsigned short  nc_localport;
int             nc_connection;
struct rx_call_descr nc_rx_cd;             /* in rx we use descriptors rather than connection */
unsigned short  nc_callflags;
nc_ct           nc_commtype;
} nct_common_service;


/*
 * Service entry (connection type for) C language access.
 */

typedef struct nct_cservice {
nct_common_service      ncc_common_service;
short                   ncc_length;
int                     (**ncc_procedures)();
unsigned short          *ncc_permissions;
unsigned long           *ncc_random_ids;
} nct_cservice;
#define ncc_link        ncc_common_service.nc_link
#define ncc_name        ncc_common_service.nc_name
#define ncc_flags       ncc_common_service.nc_flags
#define ncc_uid         ncc_common_service.nc_uid
#define ncc_gid         ncc_common_service.nc_gid
#define ncc_time        ncc_common_service.nc_time
#define ncc_hostname    ncc_common_service.nc_addr.nc_host
#define ncc_port        ncc_common_service.nc_addr.nc_port
#define ncc_protection  ncc_common_service.nc_protection
#define ncc_execution   ncc_common_service.nc_execution
#define ncc_load        ncc_common_service.nc_load
#define ncc_filename    ncc_common_service.nc_filename
#define ncc_directory   ncc_common_service.nc_directory
#define ncc_dirhostname ncc_common_service.nc_diraddr.nc_host
#define ncc_dirport     ncc_common_service.nc_diraddr.nc_port
#define ncc_protocol    ncc_common_service.nc_protocol
#define ncc_loadprotocol ncc_common_service.nc_loadprotocol
#define ncc_dirprotocol ncc_common_service.nc_dirprotocol
#define ncc_connlife    ncc_common_service.nc_connlife
#define ncc_noresponse  ncc_common_service.nc_noresponse
#define ncc_idffile     ncc_common_service.nc_idffile
#define ncc_parameters  ncc_common_service.nc_parameters
#define ncc_policy      ncc_common_service.nc_policy
#define ncc_maxconn     ncc_common_service.nc_maxconn
#define ncc_service     ncc_common_service.nc_service
#define ncc_localport   ncc_common_service.nc_localport
#define ncc_connection  ncc_common_service.nc_connection
#define ncc_cd          ncc_common_service.nc_rx_cd
#define ncc_callflags   ncc_common_service.nc_callflags
#define ncc_commtype    ncc_common_service.nc_commtype



/*
 * Exception handler descriptor.
 */

typedef struct nct_exhandler {
struct nct_exhandler    *nc_exlink;
nct_exception           nc_except;
jmp_buf                 nc_jmpbuf;
} nct_exhandler;

/*
 * Default values for timeouts to transport level.
 */

#define NC_TM_CONN      30              /* connection timer */
#define NC_TM_CONNLIFE  0               /* connection lifetime */
#define NC_TM_RETRANS   2               /* retransmit timer */
#define NC_TM_RETRANSRQ 4               /* retransmit request timer */
#define NC_TM_NRETRANS  3               /* number of retransmissions */
#define NC_TM_NORESP    10              /* no response timer */
#define NC_TM_NET       1               /* network timer */



/*
 * Externals from the common part of the library.
 */

extern int      errno;
extern char     nc_version;
extern char     nc_revision;
extern char     nc_byteorder;
extern char     nc_floattype;
extern char     *ncerrstring;       /* to control memory allocation for err strings */
extern char * print_ipaddr();

extern unsigned short swapas(short unsigned int ashort);
extern nc_32uint swapal(nc_32uint along);
extern struct nct_cservice * nc_sgetptr(int service_entry_id);
extern  nct_cservice * nc_exported_services(short unsigned int service);        /* Export list lookup routine */
extern  int nc_pdummy(void);
extern int nc_sinit(void);
extern int nc_salloc(struct nct_cservice *cd, int type);
extern int nc_sdealloc(int service_entry_id);
extern int nc_sfirst(int type);
struct nct_cservice * nc_sgetptr(int service_entry_id);
extern int nc_sfindid(struct nct_cservice *cd);


#ifdef DEBUG
#define TRACE(x) if (nctrace.tron) nctrace.x
#else
/*
 * Ignore tracing (not for the time being)
 */
#define TRACE(x) if (nctrace.tron) nctrace.x
#endif

struct nctrace{
	int tron;

	int allocation;
	int calls;
	int callmsg;
	int retmsg;
	int exceptions;
	int sockets;
	int serviceid;
	int data;
};
extern struct nctrace nctrace;


