/*
 * Copyright (C) 1990  CERN.
 *
 * xx.09.90     written by Kris Kostro and Turchanyi
 * 15.03.91     TCP interface corrected by Turchanyi Geza
 * 08.09.2000 - CS clean up verison. RTCP removed
 *
 */

#include "sysconfig.h"
#include "errtype.h"
#include "ncc.h"
#include "nc.h"



/*
   To allow the client to receive a message on a imported connection we
   have to know the service descriptor because we cannot search in the queue
   of exported services.
*/
int
nc_getclmsg(int service_entry_id, short unsigned int proc_number, _btype **rpp)
{
	struct ncct_call_header nc_lastcallh;
	nct_cservice *cd;
	register struct ncct_call_header *ncc_call;
	struct ncct_return_header *ncc_return;
	int sid, fd, fd_ret, nbytes;
	int status, rstat, length;
	short localerr = 0;
	_btype *bp, *rp;
	struct rx_call_descr call_d;
	protocol *p;

	if( (service_entry_id < 0) || ((cd = nc_sgetptr( service_entry_id )) == 0) ) {
		RETURNERR( ENCNOSERVICE, ETNCRT, "nc_sgetptr()" )
	}
	if ((p = cd->ncc_protocol) == 0) {
		RETURNERR(ENCCNOPROT, ETNCRT, "")
	}

	if (p != NC_P_RX) {
		return(-1);
	} else {
		call_d.rmsg =  (char *)&nc_lastcallh;
		call_d.rlen = sizeof(nc_lastcallh);
		call_d.timeout = 0xffffff;

		if ((status = rx_get_call(&call_d)) < 0)
			return(-1);
		if (status == 0)
			return(0);
	}

	/* Internalize the call header for later use.           */
	ncc_call = &nc_lastcallh;
	if (ncc_call->ncc_byteorder != NC_BYTEORDER) {
	    ncc_call->ncc_length = swapal( ncc_call->ncc_length );
	    ncc_call->ncc_random_id = swapal( ncc_call->ncc_random_id );
	    ncc_call->ncc_service_id = swapas( ncc_call->ncc_service_id );
	    ncc_call->ncc_procedure_number = swapas( ncc_call->ncc_procedure_number );
	    ncc_call->ncc_userid = swapas( ncc_call->ncc_userid );
	    ncc_call->ncc_groupid = swapas( ncc_call->ncc_groupid );
	    ncc_call->ncc_type = swapas( ncc_call->ncc_type );
	}

	TRACE(calls && fprintf(stderr, "nc_getcall from %s: fl %d %d %d %d len %d s_id %d typ %d pr %d\n",
		print_ipaddr(&(call_d.host)),
		ncc_call->ncc_byteorder, ncc_call->ncc_floattype, ncc_call->ncc_version,
		ncc_call->ncc_revision, ncc_call->ncc_length, ncc_call->ncc_service_id,
		ncc_call->ncc_type, ncc_call->ncc_procedure_number) );

	errno = 0;
	errtype = 0;
	length = ncc_call->ncc_length - CALL_HEADER_BYTES;

	if( (bp = nc_allocate( length + sizeof(char) )) == 0 )
		return(-1);

       {
		cd->ncc_cd.call_number = call_d.call_number;
		cd->ncc_cd.rmsg =  bp;
		cd->ncc_cd.rlen = length;
		if (length > 0 && rx_more(&(cd->ncc_cd)) < 0) {
			nc_deallocate( bp );
			return(-1);
		}
	}

	/* TRACE data */
	if (nctrace.tron && nctrace.data) {
		char *p;
		int i;

		fprintf(stderr, "call data:");
		for (i = 0, p = bp; i<length; ++i, ++p) {
			if (!(i%20)) fprintf(stderr, "\n");
			fprintf(stderr, "%2x ",*p);
		}
		fprintf(stderr, "\n");
	}
	if (ncc_call->ncc_type != NCC_MESSAGE ) {
		nc_deallocate( bp );
		RETURNERR( ENCCTYPE, ETNCRT, "unexpected msg type" );
	}

	/* Update the statistics variable */
	nc_statistics.cl_calls++;
	nc_statistics.cl_receivelen += length;

	*rpp = bp;
	return(0);

}

