/* %W% %E% */
/*
 * 01-04-89 (raw): changed getuid/getgid call for OS-9
 * xx.05.90 kk  Version 2.0
 * 07.05.90 kk  removed obsolete code and commented out code related to name
 *              server and loader (to be reviewed).
 *              replaced negative error returns by returning (-1) and setting
 *              errno/errtype
 *              added code related to the rx interface.
 * 18.06.90 kk  Temporary disabled nc_signals() call
 * 28.03.91 kk  Changed timeouts given to rx to milliseconds
 * 08.07.92 C Scafuri : added conditional include section for OS9
 * LP - 23.05.2000 - fixed in.h inclusion
 * 11.09.2000 - CS clean up verison. RTCP removed,NODAl removed
 */

#include "sysconfig.h"
#include <stdlib.h>
#if defined(LYNX2_2_1)
#include <oscalls.h>
#endif
#if defined(OS9)
  #include <types.h>
  #include <INET/in.h>
#else
  #include <sys/types.h>
  #include <netinet/in.h>
#endif
#include <string.h>
#include "errtype.h"
#include "ncc.h"
#include "nc.h"
/*#include "nodal_prot.h"*/

int ncr_imported = 0;

extern unsigned long nc_select_mask;
extern int _ncr_import;

/*
 * Import a service. Return the allocated service entry identifier.
 */



int
import_service(char *name, char *host_name, unsigned int port,
	int protection, int execution, int load, char *file_name,
	int directory, char *dirhostname, unsigned int dirport,
	protocol *p, protocol *loadp, protocol *dirp, long int connlife,
	long int noresponse, unsigned int flags, char *idffile,
	char *parameters, int policy, int maxconnections,
	unsigned int service, int *service_entry_ptr)
{
	register nct_cservice *cd;
	register int service_entry_id;
	int sid;
	int status;
	char remote_host[NCC_MAXSTR];
	int ncms_status;
	int ncms_cnt = 0;

#ifdef OS9
	int giduid;
#endif /* OS9 */

	/* Define tracing activation routine */
	settrace("tf", nc_tracefile);
	settrace("nc", nc_trace);
	/* Update the statistics variable */
	nc_statistics.import++;

	/* Allocate a connection descriptor to be used by the system    */
	/* for subsequent transactions.                                 */
	if( (cd = (nct_cservice *)nc_allocate( sizeof(nct_cservice) )) == 0 ) {
		return(-1);
	}

	/* Allocate a service entry identifier to be used by the user   */
	/* transactions.                                                */
	if( (service_entry_id = nc_salloc( cd, NC_IMPORTED )) < 0 ) {
		nc_deallocate( (char *)cd );
		return(-1);
	}
	TRACE( serviceid && fprintf(stderr, "import() sid = %d\n", service_entry_id));

	/* Insert the information given by the user into the connection */
	/* descriptor.                                                  */
	cd->ncc_name = name;
	cd->ncc_flags = flags;
#ifdef OS9
	giduid=getuid();
	cd->ncc_uid = 0xffff & giduid;
	cd->ncc_gid = giduid >> 16;
#else
	cd->ncc_uid = getuid();
	cd->ncc_gid = getgid();
#endif /* OS9 */
	tm_now( &cd->ncc_time );
	cd->ncc_hostname = host_name;
	cd->ncc_port = port;
	cd->ncc_protection = protection;
	cd->ncc_execution = execution;
	cd->ncc_load = load;
	cd->ncc_filename = file_name;
	cd->ncc_directory = directory;
	cd->ncc_dirhostname = dirhostname;
	cd->ncc_dirport = dirport;
	cd->ncc_protocol = p;
	cd->ncc_loadprotocol = loadp;
	cd->ncc_dirprotocol = dirp;
	cd->ncc_connlife = connlife;
	if (p == NC_P_RX)
		cd->ncc_noresponse = noresponse*1000;
	else
		cd->ncc_noresponse = noresponse;
	cd->ncc_idffile = idffile;
	cd->ncc_parameters = parameters;
	cd->ncc_policy = policy;
	cd->ncc_maxconn = maxconnections;
	cd->ncc_service = service;
	cd->ncc_localport = 0;
	cd->ncc_connection = NCC_NOCONNECT;
	cd->ncc_length = 0;
	cd->ncc_procedures = 0;
	cd->ncc_random_ids = 0;


	/* Things seem to be OK, so we can initialize the socket to be  */
	/* used if not done before.                                     */
	if (p == NC_P_RX) {
		/*
		 * initialise the local communication endpoint and
		 * initialise the call descriptor with the remote address
		 */
		if ((status = rx_init(&(cd->ncc_localport))) < 0) {
			nc_sdealloc( service_entry_id );
			return(-1);
		}
		cd->ncc_callflags = 0;
		if (rx_setaddress(&(cd->ncc_cd), cd->ncc_hostname, port) < 0) {
			nc_sdealloc( service_entry_id );
			return(-1);
		}
	
		/* set the nc_rx_on flag and add rx connection to the mask */
/*              not used on the client side yet
		nc_rx_fd = status;
		nc_select_mask |= (1 << status);
*/
		cd->ncc_cd.timeout = 0;
		cd->ncc_cd.flags = 0;
		/*
		 * for compatibility reasons we make an rpc call to the
		 * ncr service to fork the server if it has been requested
		 * This should lated be moved to the stub.
		 */
		if ((execution == NC_EX_DETACH) || (execution == NC_EX_ANY)) {
			if (!ncr_imported) {
				import_ncr(cd->ncc_hostname, port, noresponse);
				ncr_imported = 1;
			}
			ncr_link(cd);
			cd->ncc_port = ncr_newserv(NC_EX_NULL);
			TRACE(sockets && fprintf(stderr, "newserv returned port %d\n",cd->ncc_port));
			if (cd->ncc_port == (unsigned short)-1) {
				nc_sdealloc( service_entry_id );
				return(-1);
			}
			cd->ncc_cd.port = ntohs(cd->ncc_port);
		}
	} else {
		return(-1);	
	}

	/* The connection is open so we can set the timeouts to be      */
	/* applied in this case.                                        */
	/*if (p != NC_P_RX)*/
		if( (status = nc_tminit( p, cd )) < 0 ) {
			nc_sdealloc( service_entry_id );
			RETURNERR((-status), ETPROT, "p_ioctl()")
		}

	/* Update the variables to be returned to the client            */
	*service_entry_ptr = service_entry_id;
	return( service_entry_id );
}
