/*
 * xx.05.90 kk  Version 2.0
 * 11.09.2000 - CS clean up verison. RTCP removed
 */

#include "sysconfig.h"
#include "errtype.h"
#include "ncc.h"
#include "nc.h"


/*
 * Call a remote procedure in the remote server.
 * If inbuf is not NULL no memory will be allocated, inlen is the maximum
 * buffer size used on reading in this case.
 */

_btype *
nc_call(int mtype, int service_entry_id, unsigned int proc_number, long unsigned int random_id, _btype *outbuf, int outlen, _btype *inbuf, int inlen)
{
	register nct_cservice *cd;
	ncct_call_header *ncc_call;
	ncct_return_header ncc_return;
	int nbytes, status;
	_btype *rp;

	/* Get a pointer to the connection descriptor.                  */
	if( (service_entry_id < 0) || ((cd = nc_sgetptr( service_entry_id )) == 0) ) {
		nc_deallocate(outbuf);
		RETURNERRP((_btype*)0, ENCNOSERVICE, ETNCRT, "nc_sgetptr()" )
	}

	if (cd->ncc_protocol == NC_P_RX) {
		/* in rx protocol we send and receive in one call */

/* kkk in future we may preset header in cd and copy it */
		/* Set the protocol header.                             */
		ncc_call = (ncct_call_header *)outbuf;
		ncc_call->ncc_byteorder         = NC_BYTEORDER;
		ncc_call->ncc_floattype         = NC_FLOATTYPE;
		ncc_call->ncc_version           = nc_version;
		ncc_call->ncc_revision          = nc_revision;
		ncc_call->ncc_length            = outlen;
		ncc_call->ncc_random_id         = random_id;
		ncc_call->ncc_service_id        = cd->ncc_service;
		ncc_call->ncc_procedure_number  = proc_number;
		ncc_call->ncc_userid            = cd->ncc_uid;
		ncc_call->ncc_groupid           = cd->ncc_gid;
		ncc_call->ncc_type              = mtype;
		ncc_call->ncc_errno             = 0;

		TRACE(callmsg && fprintf(stderr, "nc_call() rpc type %d to %s on sd %d: mt %d %d v%d.%d len %d s_id %d pr %d err %d\n",
			mtype, print_ipaddr(&(cd->ncc_cd.host)), service_entry_id,
			ncc_call->ncc_byteorder, ncc_call->ncc_floattype, ncc_call->ncc_version,
			ncc_call->ncc_revision, ncc_call->ncc_length - CALL_HEADER_BYTES , ncc_call->ncc_service_id,
			ncc_call->ncc_procedure_number,ncc_call->ncc_errno ) );

		/* TRACE data */
		if (nctrace.tron && nctrace.data) {
			char *p;
			int i;

			fprintf(stderr, "nc_call(): call data:");
			for (i = 0, p = outbuf+CALL_HEADER_BYTES; i<outlen-CALL_HEADER_BYTES; ++i, ++p) {
				if (!(i%20)) fprintf(stderr, "\n");
				fprintf(stderr, "%2x ",(*p & 0xff));
			}
			fprintf(stderr, "\n");
		}

		/* Update the statistics variable
		nc_statistics.cl_calls++;
		nc_statistics.cl_sendlen += (outlen - CALL_HEADER_BYTES);*/

		/* send the call and get reply */
		cd->ncc_cd.smsg = outbuf;
		cd->ncc_cd.rmsg = (char *)&ncc_return;
		cd->ncc_cd.slen = outlen;
		cd->ncc_cd.rlen = RETURN_HEADER_BYTES;
		cd->ncc_cd.flags = cd->ncc_callflags;
		cd->ncc_cd.timeout = cd->ncc_noresponse;
		if (mtype == NCC_UNRELIABLE) {
			status = rx_unrel(&(cd->ncc_cd));
			return(0);
		} else
			status = rx_call(&(cd->ncc_cd));
		if (status < 0) {
			nc_deallocate(outbuf);
			return((char *)0);
		}
		nc_deallocate(outbuf);
		/* Internalize the return header for later use.                 */
		if (ncc_return.ncc_byteorder != NC_BYTEORDER) {
			ncc_return.ncc_length = swapal( ncc_return.ncc_length );
			ncc_return.ncc_errtype = swapas( ncc_return.ncc_errtype );
			ncc_return.ncc_errcode = swapas( ncc_return.ncc_errcode );
		}
		if (ncc_return.ncc_errcode != 0)
			errno = ncc_return.ncc_errcode;

		TRACE(retmsg && fprintf(stderr, "nc_call(): reply : mt %d %d v%d.%d len %d err_t %d err %d\n",
			ncc_return.ncc_byteorder, ncc_return.ncc_floattype, ncc_return.ncc_version,
			ncc_return.ncc_revision, ncc_return.ncc_length  - RETURN_HEADER_BYTES, ncc_return.ncc_errtype,
			ncc_return.ncc_errcode) );

		if( (cd->ncc_cd.rlen = (ncc_return.ncc_length - RETURN_HEADER_BYTES)) > 0 ) {
			if ((rp = nc_allocate( cd->ncc_cd.rlen )) == (char *)0)
				return((char *) 0);
			/* read the outstanding data.        */
			cd->ncc_cd.rmsg = rp;
			if( rx_more(&(cd->ncc_cd)) < 0 ) {
				nc_deallocate(rp);
				return((char *)0);
			}
			/* TRACE data */
			if (nctrace.tron && nctrace.data) {
				char *p;
				int i;

				fprintf(stderr, "nc_call(): return data:");
				for (i = 0, p = rp; i<cd->ncc_cd.rlen; ++i, ++p) {
					if (!(i%20)) fprintf(stderr, "\n");
					fprintf(stderr, "%2x ",*p);
				}
			fprintf(stderr, "\n");
			}
		} else {
			/* Just allocate a dummy which the stub can deallocate. */
			rp = nc_allocate( 1 );
		}
		/* Update the statistics variable
		nc_statistics.cl_receivelen += cd->ncc_cd.rlen;*/

		/* Check the status to see if everything is OK.                 */
		if( ncc_return.ncc_errtype != 0 ) {
			nc_deallocate(ncerrstring); /* keep only last err string */
			internalize_string( &ncerrstring, rp );
			nc_deallocate( rp );
			RETURNERRP((_btype*)0, ncc_return.ncc_errcode,
				(ncc_return.ncc_errtype + ETOFSREM), ncerrstring )
		}

		/* Return the buffer to the calling stub.                       */
		return( rp );
	} else {

		return( 0 );

	}
}
