/*
 * Copyright (C) 1986 by CERN.
 *
 * 19.05.90     First written by Kris Kostro (nc version 2)
 * 13.09.90 kk  To allow polling for messages the last message can be pointed
 *              to by the global pointer nc_last_msg
 */

#include "sysconfig.h"
#include "errtype.h"
#include "ncc.h"
#include "nc.h"

extern _btype *nc_last_msg;
extern nct_cservice *nc_lastcd;
extern int nc_last_proc;

/*
 * Stub interface for getting messages. Calls down to nc_getcall after some checks.
 */

int
nc_getmsg(int service_entry_id, unsigned int proc_number, _btype **rpp)
{
	int sid, message_id;
	long timeout = 0xfffffff;

	if (nc_wait_msg(timeout, NC_P_FIRST, &sid, &message_id) < 0)
		return(-1);
	if ( message_id != proc_number || service_entry_id != sid) {
		if (nc_last_msg != 0)
			nc_deallocate(nc_last_msg);
		RETURNERR(ENCCBADPR, ETNCRT, "unexpected message")
	}
	*rpp = nc_last_msg;
	nc_last_msg = 0;
	return(0);
}

int
nc_wait_msg(long int timeout, protocol *p, int *service_entry_id, int *message_id)
{
	int sid;
	register nct_cservice *cd;

	if (p == NC_P_FIRST) {
		/* Check if any services are exported at all.           */
		if( (sid = nc_sfirst( NC_EXPORTED )) < 0 ) {
			return(-1);
		}

		/* Get the current protocol to be used. For now just            */
		/* use the protocol given in the first service.                 */
		if( (cd = nc_sgetptr( sid )) == 0 ) {
			RETURNERR( ENCNOSERVICE, ETNCRT, "nc_sgetptr()" )
		}
		if ((p = cd->ncc_protocol) == 0) {
			RETURNERR(ENCCNOPROT, ETNCRT, "")
		}
	}

	if (nc_last_msg != 0)
		return(0);

	TRACE(calls && fprintf(stderr, "nc_wait_msg():  prot %s tmo %ld\n",
		p->p_name, timeout));
	if (nc_getcall(p, timeout) == 0) {
		RETURNERR(ENCCBADPR, ETNCRT, "procedure call")
	}
	*message_id = nc_last_proc;
	*service_entry_id = nc_sfindid(nc_lastcd);
	return(0);
}
