/*
 * Interface for asynchronous client calls.
 * CS - 07.09.2000 : general heavy clenaup
 *                   removed all old dead code: RX_KERNEL etc... 
 */

#include "allincl.h"


rx_get_reply(struct rx_call_descr *cd)
{
	/*
	   Get reply to the asynchronous call. The cd should contain the
	   call number filled in by rx_call().
	   rx_get_reply() returns length of the reply (or part of it) read or
	   (-1) if it failed. In case of failure errno, errtype and errstring
	   will be set. It returns 0 if there is no return message.
	   If only part of the message has been read then rx_more() can be
	   used to read the remainder.
	   rx_wait() can be used to select on multiply replies and to wait
	   with timeout.
	*/
	long status;
	register struct association *a;
	register struct inout_pkt *pk;

	TRACE(calls && fprintf(stderr, "rx_get_reply(): ") && print_cd(cd));

	/* find the reply in the queue */

	for (a = (struct association *)RXHEAD; a != (struct association *)0 &&
		!(a->type == RX_CLIENT && a->call_number == cd->call_number);
		a = (struct association *)a->link)
	    ;

	if (a == (struct association *)0)
		RETURNERR(EBADCALLN, ETRX, "no match for call no")
	TRACE(assoc && fprintf(stderr, "rx_get_reply(): found assoc at %x\n", (long)a));


	if ( (char *)a->in_pkts.head == NULL)
		return(0);

	/* we found reply */
	a->rmsg = cd->rmsg;
	a->rlen = cd->rlen;

	while ((char *)(pk = (struct inout_pkt *)a->in_pkts.head) != NULL) {
		if (a->rlen < pk->len) {
			/* only part of packet fits in */
			bcopy(pk->data + sizeof(struct rx_hdr), a->rmsg, a->rlen);
			a->readp = pk->data + sizeof(struct rx_hdr) + a->rlen;
			pk->len -= a->rlen;
			return(cd->rlen); /* we have read as much as requested */
		} else {
			/* copy the whole packet and free it */
			bcopy(pk->data + sizeof(struct rx_hdr), a->rmsg, pk->len);
			a->rmsg += pk->len;
			a->rlen -= pk->len;
			rx_free_packet(q_rhead(&a->in_pkts));
		}
	}
	return(cd->rlen - a->rlen);

}


int
rx_wait(long unsigned int *call_number, long int timeout)

/*
 * This call is useful for applications which do asynchronous RPC calls.
 * It returns one if there is a message (reply or call) queued or zero if
 * timeout has expired. If timeout is zero, return is immediate. The call will
 * block until a message arrived or until the timeout (in seconds). Timeout
 * of 0 can be used for polling. If call_number is not 0, then the call will
 * wait for the reply matching the specified call number.If call_number == 0
 * and a reply has been found, the call number will be set to the call number
 * matching the first asynchronous reply in the queue.
 * To select calls rx_get_call() should be used.
 */
{

	register struct association *a;


	TRACE(calls && fprintf(stderr, "rx_wait(%d, %ld)\n", *call_number, timeout));


	/* walk through the association list looking for reply */
	if (*call_number == 0) {
	    RX.call_tmo = (timeout/RX_MS_TICK)*RX_MS_TICK;
	    if (!(RX.call_tmo))
		    RX.call_tmo = RX_MS_TICK;
	    for(;;) {

		for (a = (struct association *)RXHEAD; a != (struct association *)0 &&
		    !((a->type == RX_CLIENT) && (a->state == RECEIVED) && a->in_pkts.head != (struct queue_item *)0);
		    a = (struct association *)a->link)
		;

		if (a != (struct association *)0) {
			*call_number = a->call_number;

			TRACE(assoc && fprintf(stderr, "rx_wait(): found assoc at %x\n", (long)a));
			return(1);
		}
		if (RX.call_tmo == 0)
			return(0);
		/* block if none found */

		AWAIT_ANY_INTERRUPT;

	    }
	} else {
	    /* call number not 0 */
		/* first find the matching association */

		for (a = (struct association *)RXHEAD; a != (struct association *)0 &&
		    !(a->type == RX_CLIENT && a->call_number == *call_number);
		    a = (struct association *)a->link)
		;
		if (a == (struct association *)0) {
			/* not found, return an error */
			RETURNERR(EBADCALLN, ETRX, "no match for call no")
		}

		TRACE(assoc && fprintf(stderr, "rx_wait(): found assoc at %x\n", (long)a));

		for (;;) {
			/* the timeout from call is still valid */
			if (a->state == RECEIVED)
				return(1);
			if (a->state == ERROR_STATE)
				RETURNERR(a->error, ETRX, "")

			/* block if none found */
			 a->w_intr = 1;  /* for AWAIT_INTERRUPT */
			 AWAIT_INTERRUPT(a);
		}
	}

}


