/*
 * Interface for server to get a new call.
 */
/* 23.03.95 kk	set the minimum timeout in rx_get_call to RX_MS_TICK
		(instead of 5sec if less than 100ms)
 * CS - 07.09.2000 : general heavy clenaup
 *                   removed all old dead code: RX_KERNEL etc...		
*/

#include "allincl.h"


int

rx_get_call(struct rx_call_descr *cd)
{
/*
 * Get any call message. The address and length of buffer are given by cd.
 * The call returns the actual length read and cd will be filled with the
 * call number. If only part of the message has been read then rx_more()
 * can be used to read the remainder.
 * The call times out after the number of seconds specified in cd->timeout
 * and returns zero. Timeout of zero can be used for polling.
 */

	register struct association *a;
	register struct inout_pkt *pk;

	TRACE(calls && fprintf(stderr, "rx_get_call(): ") && print_cd(cd));
	if (cd->timeout && cd->timeout < RX_MS_TICK)
		/* This is a repetitive call and low timeouts cause overhead */
		cd->timeout = RX_MS_TICK;
	RX.call_tmo = (cd->timeout/RX_MS_TICK)*RX_MS_TICK;
	if (!(RX.call_tmo))
		RX.call_tmo = RX_MS_TICK;

	/* find the first call in the queue */
	for(;;) {

	    for (a = (struct association *)RXHEAD; a != (struct association *)0 &&
		    !(a->type == RX_SERVER &&
			(a->state == RECEIVED || a->state == SENT) &&
			a->in_pkts.head != (struct queue_item *)0);
		    a = (struct association *)a->link)
		;

	    if (a != (struct association *)0) {
		/* We found a partial message */
		break;
	    }
	    if (RX.call_tmo == 0) {
		return(0);
	    }
	    /* block if none found */

	    AWAIT_ANY_INTERRUPT;
	}

	/* we found a new call */

	RX.call_tmo = 0;
	rx_glob.call_number ++;         /* to avoid message reply with the same call no */

	/* place the association at the end of the list to have round-robin */

	q_delete(&(rx.ass), (struct queue_item *)a);
	q_atail(&(rx.ass), (struct queue_item *)a);

	cd->host = a->addr_peer.host; cd->port = a->addr_peer.port;
	cd->call_number = a->call_number;
	a->rmsg = cd->rmsg;
	a->rlen = cd->rlen;

	while ((char *)(pk = (struct inout_pkt *)a->in_pkts.head) != NULL) {
		if (a->rlen < pk->len) {
			/* only part of packet fits in */
			bcopy(pk->data + sizeof(struct rx_hdr), a->rmsg, a->rlen);
			a->readp = pk->data + sizeof(struct rx_hdr) + a->rlen;
			pk->len -= a->rlen;
			return(cd->rlen); /* we have read as much as requested */
		} else {
			/* copy the whole packet and free it */
			bcopy(pk->data + sizeof(struct rx_hdr), a->rmsg, pk->len);
			a->rmsg += pk->len;
			a->rlen -= pk->len;
			rx_free_packet(q_rhead(&a->in_pkts));
		}
	}
	a->prot_flags ^= NOREPL_FLG;
	return(cd->rlen - a->rlen);

}
