/* %W% %E% */
/*
 * 09/11/88 raw: added code for case when uname gives empty hostname
 * 02/03/90 raw: added code to scan /dd/ETC/hosts for OS-9
 * 17/04/90 raw: replaced incorrect use of "rindex" by "index" for OS-9;
 *               return a positive value in errno (if any)
 *
 * Get the host name of own system. This function is necessary on Unix System V
 * because no gethostname() is provided. For Berkeley Unix this function
 * is provided.
 * The uname function is used to obtain the hostname. In case that uname
 * returns an empty string for hostname, the file /etc/hosts is scanned for
 * an entry with the alias "localhost", and the corresponding hostname
 * (second field on that line) is returned.
 *
 * input:  namelen  maximal length of hostname
 * output: name     pointer to the hostname string
 * return: 0        if no errors
 *         -1       otherwise
 *
 * Authors: I.Kristiansen (CERN), R.Wilcke (ESRF)
 */

#include "sysconfig.h"
#if !defined(OS9)
#include "sys/utsname.h"
#include <string.h>
#include "stringutil.h"
#else
#define strchr index
#include <ctype.h>
#include <strings.h>
#endif
#include "errno.h"
#include <stdio.h>

extern int errno;

gethostname( name, namelen )
   char *name;
   int namelen;
{

/* Maximum line length for reading hostfile */
#define MAXBUF 256

#if !defined(OS9)
   struct utsname locsys;
#endif
   FILE *stream;
   int status, rdstat, length, buflen, loclen;
   char buf[MAXBUF];

   /*
    * Hostfile definition: /etc/hosts for UNIX,
    *                      /dd/ETC/hosts for OS-9
    */
#if !defined(OS9)
   static char hostfile[]="/etc/hosts";
#else
   static char hostfile[]="/dd/ETC/hosts";
#endif

   static char locstr[]="localhost";   /* Identifies local host in hostfile */
 
   /* Separators in /etc/hosts: Any combination of blanks and tabs */
   static char sepstr[]=" 	";     

   char *bufptr, *hostnm;

   /*
    * Get the system information record containing the host name.
    * For OS-9, we skip the uname-command and try directly the hostfile.
    */
#if !defined(OS9)
   if( (status = uname( &locsys )) < 0 ) {
      return( -1 );
   }

   /* Check if the nodename is not an empty string.      */
   if( locsys.nodename[0] != 0) {

      /* Check if the space provided by the user is sufficient.   */
      if( (length = strlen( locsys.nodename )) >= namelen ) {  
         errno = EFAULT;
         return( -1 );
      }

      /* Copy the name into the space provided by the user.      */
      strcpy( name, locsys.nodename );  

   } else {
#endif
   /*
    * Hostname from uname was empty. Get the hostname from hostfile.
    */

      /* Open hostfile. Return -1 if this fails.          */
      loclen=strlen(locstr);
      if((stream = fopen(hostfile,"r")) == NULL)
      {
         return(-1);
      }

      /*
       * Scan hostfile. Discard lines starting with a '#'
       * (comment lines). Search the ones passing this test for
       * the string "localhost" (contained in locstr). Stop scanning
       * at the first line thus found.
       */
      while((rdstat = (int)fgets(buf,MAXBUF,stream)) != 0) {
         if(buf[0] != '#') {
            for(bufptr=buf; (bufptr=strchr(bufptr,locstr[0]))!=NULL; bufptr++) {
               if(strncmp(bufptr,locstr,loclen) == 0)
                  break;
            }
            if(bufptr != NULL)
               break;
         }
      }
      fclose(stream);

      /*
       * If no line with the string "localhost" was found, return -1.
       * Otherwise, read the second string in the line found: this is
       * the host name (see definition of hostfile in the TCP/IP
       * manuals).
       */
      if(rdstat == 0) {
         errno= 1;
         return(-1);
      }

#if !defined(OS9)
      strtok(buf,sepstr);
      hostnm=strtok(NULL,sepstr);
#else
#define EFAULT 14
      for(bufptr=buf;isspace(*bufptr) == 0; bufptr++);
      for(; isspace(*bufptr) != 0; bufptr++);
      hostnm=bufptr;
      for(; isspace(*bufptr) == 0; bufptr++);
      *bufptr=0;
#endif

      /* Check if the space provided by the user is sufficient.   */
      if((length=strlen(hostnm)) >= namelen) {
         errno=EFAULT;
         return(-1);
      }

      /* Copy the name into the space provided by the user.      */
      strcpy(name,hostnm);

#if !defined(OS9)
   }
#endif

   /* Successful return.                                         */ 
   return(0);
}
