/*
 * Adapted from iptst to do the same measurements for NC. The below comments
 * refer to iptst.
 *
 * The main (client) program to test functionality and performance of tcp/ip
 * protocols (including rex rpc protocol). The main goal is to estimate
 * fixed and variable costs of an RPC protocol. Among other the following
 * estimates are possible:
 *      - the minimal roundtrip time
 *      - the maximum throughput
 *      - cost of the server fork(s)
 *      - cost of the write/read system calls
 *      - cost of sendto/recvfrom as compared to connect+write/read
 *      - cost of the host name resolution
 *      - cost of the connection establishement
 *
 * The following tests (opereations)
 * are possible:
 * -os : source (send)
 * -ot : transaction (echo data sent)
 *
 * Call tst without arguments to get usage explaination.
 */

/*
 * C Scafuri : include section adapted for OS9 cross compilation
 */

#include <sysconfig.h>

#if defined(OS9)
#include <types.h>
#include <time.h>
#else
#include <sys/types.h>
#if (defined(LAI_TCP) || defined(AIX))
#include <sys/select.h>
#else
#include <sys/time.h>
#endif
#endif
#include <stdio.h>
#include "ntst.h"
#include <signal.h>
#include "ncc.h"
#include "rx.h"

extern long atol();
extern char *malloc();
extern void intrit();
extern struct rx_stats rx_stats;
extern int network_lossage;

main(argc, argv)
	int argc;
	char **argv;
{
/*
 * Variables for intercepting program call parameters.
 */
	char *argv0;
	char *name = 0;
	unsigned short lport = 0;
	char operation = 't';
	int nforks = 0;                     /* n of forks by the server */
	long data_size = 100;      /* data size (bytes) to be transferred */
	long block_size = 100;     /* max block size read/written */
	int mloop = 1, loop = 1;
	int verbose = 0;
	int protocol = P_RX;

	/* database recording */
	char dbfile[30];
	FILE *dbf;
	char myhost[30];
	int record = 0;
	char *mode = "nomode";
	char *gate = "E";
	char *special = "";
	char *comment = "";
	struct hostent *hp;
	int stat;

	char *buf;
#ifdef RX_PROTO
	struct rx_call_descr cd;
#endif

	int sd;
	int status;
	long offset = 0;
	short limit = 0;
	static struct timeval start_time = {0, 0}, end_time = {0, 0};
	long elapsed;
	int i, j, k;
	long bytes_total, current_length, bytes_read, bytes_written;

	/* to deal with UDP packet loss */
	int select_errs = 0, select_tmouts = 0;

	nc_sequence seq;

	buf = malloc(MAXDSIZE);
/*
 * Read call parameters.
 */
	argv0 = argv[0];
	name = *++argv;
	if (--argc < 0)
		goto usage;
	while (--argc > 0 && **++argv == '-') {
		switch ((*argv)[1]) {
		    case 'q':
			if (--argc > 0)
				lport = atoi(*++argv);
			break;
		    case 'f':
			if(--argc > 0)
				nforks = atoi(*++argv);
			break;
		    case 'l':
			if (--argc > 0)
				network_lossage = atoi(*++argv);
			break;
		    case 'd':
			if (--argc > 0) {
				data_size = atol(*++argv);
			}
			break;
		    case 'b':
			if (--argc > 0) {
				block_size = atol(*++argv);
			}
			break;
		    case 'N':
			if (--argc > 0)
				mloop = atoi(*++argv);
			break;
		    case 'n':
			if (--argc > 0)
				loop = atoi(*++argv);
			break;
		    case 'o':
			operation = (*argv)[2];
			break;
		    case 'g': /* gateway info */
			if (--argc > 0)
				gate = *++argv;
			break;
		    case 'm': /* enables recording used in mea script */
			record = 1;
			if (--argc > 0)
				mode = *++argv;
			break;
		    case 's': /* special info (select, gethost etc) */
			if (--argc > 0)
				special = *++argv;
			break;
		    case 'c': /* comments */
			if (--argc > 0)
				comment = *++argv;
			break;
		    case 't': /* UDP transaction timeout */
			if (--argc > 0)
			break;
		    case 'v': /* verbose */
			verbose = 1;
			break;
		    case 'z': /* rx trace */
			if (--argc > 0)
				rx_trace(*++argv);
			break;
		    default:
			goto usage;
		}
	}
	if (argc != 0) {
usage:
		fprintf(stderr, "Usage: %s hostname \
[-q local port] [-P{u|t|x|U}] [-o{t|s}]\n\
\t[-d data_size] [-b block_size] [-N mainloop] [-n loop] [-f nforks]\n\
\t[-m modeword] [-g gateword] [-s specialword] [-c commentword]\n\
\t[-t select tmout] [-v]\n", argv0);
		fprintf(stderr, "\t\tu - 'connectionless' UDP,   U - UDP with connect,   T - TCP\n\
\t\tt - transaction mode,   s - source mode\n");
		exit(1);
	}

	if (block_size > data_size)
		block_size = data_size;
	if (data_size > MAXDSIZE)
		data_size = MAXDSIZE;

	bytes_read = bytes_written = 0;
#ifdef DEBUG
	fprintf(stderr, "ntst: prepared request size %d bsize %d rep %d nforks %d\n",
		data_size, block_size, loop, nforks);
#endif

	signal(SIGINT, intrit); /* to interrupt system calls and print */

	tm_now(&start_time);
/* the big loop for connection measurement */
    for (j = 0; j < mloop; j++) {
		if (import_ntst(name) < 0) {
			nc_perror("ntst: import_ntst()");
			goto printit;
		}

	switch (operation) {
	    case 't': /* transaction */
		for (i = 0; i < loop; ++i) {
			seq.length = data_size/4;
			seq.sequence = (idl_long_int *)buf;
			if (echo(&seq) < 0) {
				nc_perror("ntst: echo()");
				goto printit;
			}
			nc_deallocate((char *)seq.sequence);
			bytes_written += data_size;
			bytes_read += data_size;
#ifdef DEBUG
			fprintf(stderr, "ntst: echo returned %d bytes\n", data_size);
#endif
		}
		break;
	    case 'a': /* asynchronous transaction */
		/* it is the same as transaction but we do send and receive */

		for (i = 0; i < loop; ++i) {
#ifdef DEBUG
			fprintf(stderr, "ntst: echo send returned \n");
#endif

			bytes_written += data_size;
			bytes_read += data_size;
#ifdef DEBUG
			fprintf(stderr, "ntst: echo get returned %d bytes\n", data_size);
#endif
		}
		break;
	    case 's': /* source with status return */
		for (i = 0; i < loop; ++i) {
			seq.length = data_size/4;
			seq.sequence = (idl_long_int *)buf;
			if (src_ack(&seq) < 0) {
				nc_perror("ntst: echo()");
				goto printit;
			}
			bytes_written += data_size;
#ifdef DEBUG
			fprintf(stderr, "ntst: src_ack returned\n");
#endif
		}
		break;
	    case 'n': /* source without status return */
		for (i = 0; i < loop; ++i) {
			seq.length = data_size/4;
			seq.sequence = (idl_long_int *)buf;
			if (src_noack(&seq) < 0) {
				nc_perror("ntst: echo()");
				goto printit;
			}
			bytes_written += data_size;
#ifdef DEBUG
			fprintf(stderr, "ntst: src_noack returned\n");
#endif
		}
		break;
	    default:
		fprintf(stderr, "invalid operation: %d\n",operation);
		goto usage;
	}

	if (verbose)
		fprintf(stderr,"-");
    } /* main loop */

printit:
	if (verbose)
		fprintf(stderr,"\n");
	tm_now(&end_time);
	elapsed = (float)((end_time.tv_sec*1000000 + end_time.tv_usec) -
		(start_time.tv_sec*1000000 + start_time.tv_usec))/1000;
#ifdef SYSVtime
	if (elapsed == 0)
		elapsed = 1000;
	if (elapsed < 10000) {
		printf("Elapsed time less then 10 sec. !!!!!\n");
		printf("Error can be as high as %d%%\n\n",
			100/(elapsed/1000));
	}
#endif
	printf("msec\trepet\tKB data\tbl size\t\tKB/sec\tms/trans\n\n");
	printf("%ld\t%d\t%ld\t%ld\t\t%.3f\t%.3f\n", elapsed, mloop*loop,
	(bytes_read+bytes_written)/1000, block_size,
	((float)(bytes_read + bytes_written))/elapsed,
	(float)(elapsed)/(mloop*loop));
	if (select_errs | select_tmouts)
		printf("select: errors %d timeouts %d\n",select_errs, select_tmouts);
	printf("\n");
	rx_statistics(&rx_stats);

	/* append measurements to the database file */
	if (record) {
		if (gethostname(myhost, 30) == -1)
			PRINTE("ntst: gethostname()");
		strcpy(dbfile, "db.");
		strcat(dbfile, myhost);
		if ((dbf = fopen(dbfile, "a")) == NULL)
			PRINTE("ntst: fopen()");
		fprintf(dbf,"%s %s\t %s %.3f %.3f \
%d %d %d %d %d \
%ld %d %d %s %ld %d ;%s ;%s\n",
			myhost, name, mode,
			((float)(bytes_read + bytes_written))/elapsed,
			(float)elapsed/(mloop*loop),
			protocol, operation, mloop, loop, elapsed,
			data_size, block_size,
			nforks, gate, start_time.tv_sec, select_tmouts,
			special, comment);

	}
}

void intrit(x)
	int x;
{
	printf("interrupted by SIGINTR\n");  
	exit(1);
}
