/*
 *	main.c
 *		Main client program for 'svt' -
 *		Simple Variable Test
 *
 *	Modified:
 *	17 Feb 89, PSA
 *		Added status to floating tests.
 */

#include <stdio.h>
#include <math.h>
#include <ncc.h>

#include "svt_nc.h" 
#include "svt.h"    
                               
#include "svt_defs.h"

/* Internals - */
void usage();

/* Externals - */
extern void help(), nc_err(), not_yet(), intern_fault();
extern int  errno;

char    *host;
float version= VERSION; 


main( argc, argv )
	int		argc;
	char	*argv[];
{
   int      err_cnt=0, tot_err_cnt=0; /* Total error count */
   int      port=PORT;                /* Port number for this service */
   int      sid;                      /* Connection id */
   int      i;  
   short    status;
   char  uci, uco, ucr;
   short          si,  so,  sr; 
   int            ii,  io,  ir;
   long           li,  lo,  lr;
   unsigned short usi, uso, usr; 
   unsigned int   ui,  uo,  ur;
   unsigned long  uli, ulo, ulr;
   float          fi,  fo,  fr; 
   float_U        Ufi, Ufo, Ufr;
   double         dfi, dfo, dfr;
   double         diff;

   /* Check for possible option */
   if (argc < 2) {
         usage( argv[0] );
         help();
         exit(0);
   }

   host = argv[1];
   for(i=2; i<argc; i++) if ( argv[i][0] == '-' )
   {
      if (strcmp(argv[i], "-host") == 0)
      {
	if (i+1 < argc) host = argv[i+1];
      } 
      else if (strcmp(argv[i], "-port") == 0)
      {
	if (i+1 < argc) port = atoi(argv[i+1]);
      }
      else if (strcmp(argv[i], "-rxtrace") == 0)
      {
	if (i+1 < argc) rx_trace(argv[i+1]);
      }
      else if (strcmp(argv[i], "-nctrace") == 0)
      {
	if (i+1 < argc) nc_trace(argv[i+1]);
      }
      else if (strcmp(argv[i], "-help") == 0)
      {
        help();
        exit(0);
      }
      else 
      {
         usage( argv[0] );
         help();
         exit(0);
      }
   }

   printf("'%s' v %.2f: test NC/RPC version %d.%d for simple variables to '%s'\n",
	    SERVICE_NAME, version, nc_version, nc_revision, host );
   if ((sid = IMPORT( host, port)) < 0)
   {
	   nc_perror("import");
      exit(1);
   }
   fprintf(stderr, "Test 'boolean' - ");
   uci = FALSE; uco = 0; err_cnt = 0;
   if ( (ucr = charx(uci, &uco)) == (boolean)(-1))
	nc_perror("RPC charx() failed");
   else {
      if (ucr == TRUE)
      {
	 fprintf(stderr, "\n\treturn value failed = %d (%d)", ucr, !uci);
	 err_cnt++;
      }
      if (uco == FALSE)
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %d (%d)", uco, uci);
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%d) ok\n", uci);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }

   fprintf(stderr, "Test 'unsigned char' - ");
   uci = MUOCT / 2; uco = 0; err_cnt = 0;
   if ( (ucr = char2(uci, &uco)) == (character)(-1))
	nc_perror("RPC char2() failed");
   else {
      if (ucr != uci)
      {
	 fprintf(stderr, "\n\treturn value failed = %d (%d)", ucr, uci);
	 err_cnt++;
      }
      if (uco != (char)(2 * uci))
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %d (%d)", uco, (2*uci));
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%u) ok\n", uci);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }
    
   fprintf(stderr, "Test 'short integer' - ");
   si = MSHRT / 2; so = 0; err_cnt = 0;
   if ( (int)(sr = short2(si, &so)) == -1)
	nc_perror("RPC short2() failed");
   else {
      if (sr != si)
      {
	 fprintf(stderr, "\n\treturn value failed = %d (%d)", sr, si);
	 err_cnt++;
      }
      if (so != 2 * si)
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %d (%d)", so, (2*si));
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%d) ok\n", si);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }

   fprintf(stderr, "Test 'long integer' - ");
   li = MLONG/2; lo = 0; err_cnt = 0;
   if ( (lr = long2(li, &lo)) == (long)-1)
	nc_perror("RPC failed");
   else {
      if (lr != li)
      {
	 fprintf(stderr, "\n\treturn value failed = %ld (%ld)", lr, li);
	 err_cnt++;
      }
      if (lo != 2 * li)
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %ld (%ld)", lo, (2*li));
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%ld) ok\n", li);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }


   fprintf(stderr, "Test 'short cardinal' - ");
   usi = MUSHRT/2; uso = 0; err_cnt = 0;
   if ( (int)(usr = ushort2(usi, &uso)) == (cardinal)-1)
	nc_perror("RPC failed");
   else {
      if (usr != usi)
      {
	 fprintf(stderr, "\n\treturn value failed = %u (%u)", usr, usi);
	 err_cnt++;
      }
      if (uso != 2 * usi)
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %u (%u)", uso, (2*usi));
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%u) ok\n", usi);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }
 

   fprintf(stderr, "Test 'long cardinal' - ");
   uli = (unsigned long)MULONG / 2; ulo = 0; err_cnt = 0;
   if ( (ulr = ulong2(uli, &ulo)) == (long_cardinal)-1)
	nc_perror("RPC failed");
   else {
      if (ulr != uli)
      {
	 fprintf(stderr, "\n\treturn value failed = %lu (%lu) [ = %lx (%lx)]", ulr, uli, ulr, uli);
	 err_cnt++;
      }
      if (ulo != 2 * uli)
      {
	 fprintf(stderr, "\n\t'out' parameter failed = %lu (%lu)", ulo, (2*uli));
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%lu) ok\n", uli);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }


   fprintf(stderr, "Test 'short unspecified' - ");
   usi = USSPI; uso = 0; err_cnt = 0;
   if ( (int)(usr = usspx(usi, &uso)) == (unspecified)-1)
	nc_perror("RPC failed");
   else {
      if (usr != usi)
      {
	 fprintf(stderr, "\n\treturn value did not match = 0x%x (0x%x) [Probably different CPU architechture]", usr, usi);
	 err_cnt++;
      }
      if (uso != usi)
      {
	 fprintf(stderr, "\n\t'out' parameter did not match = 0x%x (0x%x) [Probably different CPU architechture]", uso, usi);
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(0x%x) ok\n", usi);
      else
      {
	    tot_err_cnt += err_cnt;
	    fprintf(stderr, "\n");
      }
   }

   fprintf(stderr, "Test 'long unspecified' - ");
   uli = ULSPI; ulo = 0; err_cnt = 0;
   if ( (int)(ulr = ulspx(uli, &ulo)) == -1)
	nc_perror("RPC failed");
   else {
      if (ulr != uli)
      {
	 fprintf(stderr, "\n\treturn value did not match = 0x%lx (0x%lx) [Not reciprocal byteswapping]", ulr, uli);
	 err_cnt++;
      }
      if (ulo != uli)
      {
	 fprintf(stderr, "\n\t'out' parameter did not match = 0x%lx (0x%lx) [Not reciprocal byteswapping]", ulo, uli);
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(0x%lx) ok\n", uli);
      else
      {
	 tot_err_cnt += err_cnt;
	 fprintf(stderr, "\n");
      }
   }


   fprintf(stderr, "Test 'float' - ");
   fi = FNUMB; fo = 0.0; err_cnt = 0; status = 0;
   Ufi.f = fi; 
/*
   fprintf(stderr, "Orig. pattern=0x%x 0x%x 0x%x 0x%x\n", Ufi.c[0], Ufi.c[1], Ufi.c[2], Ufi.c[3]);
*/
   if ( (int)(fr = float2( fi, &fo, &status )) == -1)
	nc_perror("RPC failed");
   else {
      if (status != 0)
      {
	 fprintf(stderr, "status=%d\n", status);
	 fprintf(stderr, "\n\tThe test server did not recognize the value of the 'in' parameter.\n");
	 fprintf(stderr, "\tThere is probably a problem with the machine conversion of type 'float'.\n");
		   err_cnt++;
	   }
       if (( diff = fabs((double)(fi - fr))) > FDIFF_LIMIT )
       {
	  fprintf(stderr, "\n\treturn value failed =\n\t %40.17f\n\t(%40.17f)\n\tdelta=%40.17f", fr, fi, diff);
	  err_cnt++;
       }
       if (( diff = fabs((double)(fo - 2 * fi))) > FDIFF_LIMIT )
       {
	  fprintf(stderr, "\n\t'out' parameter failed =\n\t %40.17f\n\t (%40.17f)\n\tdelta=%40.17f", fo, (2 * fi), diff);
	  err_cnt++;
       }
       if (err_cnt == 0) fprintf(stderr, "(%g) ok\n", fi);
       else
       {
	  tot_err_cnt += err_cnt;
	  fprintf(stderr, "\n");
       }
   }


   fprintf(stderr, "Test 'double float' - ");

   dfi = DNUMB; dfo = 0.0; err_cnt = 0; status = 0;
   if ( (int)(dfr = double2( dfi, &dfo, &status )) == -1)
	nc_perror("RPC failed");
   else {
      if (dfr  == (double)0.0)
      {
	 fprintf(stderr, "\n\tThe server did not recognize the value of the 'in' parameter.\n");
	 fprintf(stderr, "\tThere is probably a problem with the machine conversion of type 'double'.\n");
	 err_cnt++;
      }
      if (( diff = fabs((double)(dfi - dfr))) > DDIFF_LIMIT )
      {
	 fprintf(stderr, "\n\treturn value failed =\n\t %40.17f\n\t(%40.17f)\n\tdelta=%40.17f", dfr, dfi, diff);
	 err_cnt++;
      }
      if (( diff = fabs((double)(dfo - 2 * dfi))) > DDIFF_LIMIT )
      {
	 fprintf(stderr, "\n\t'out' parameter failed =\n\t %40.17f\n\t(%40.17f)\n\tdelta=%40.17f", dfo, (2 * dfi), diff);
	 err_cnt++;
      }
      if (err_cnt == 0) fprintf(stderr, "(%g) ok\n", dfi);
      else
      {
	 tot_err_cnt += err_cnt;
	 fprintf(stderr, "\n");
      }
   }
}

 
void	usage( argv0 )
	char *argv0;
{
printf( "Usage: %s hostname [-options]\n\
Valid options are (defaults in parenthesis):\n\
\t-port <port>         specify tcp/ip port (nc_tst);\n\
\t-help  	       provide some help for other commands;\n\
\n", argv0 );
} 


