/*
 * svtd.c
 *
 * Original:
 *    19 Dec 88, PSA
 * Modified:
 *    17 Feb 89, PSA
 *	 Added status to floating tests.
 *    27 Jan 89, PSA
 *       Problem with convincing XENIX to compare floats.
 *
 */

#include <stdio.h>
#include <math.h>

#include "svt_nc.h"
#include "svt.h"
  
/* Internals - */
void usage();

/* Externals - */
extern void help(), not_yet(), intern_fault();
extern int rtcp_traceread;

int port = PORT;

main( argc, argv )
	int argc;
	char **argv;
{
	int i, sid, status;
 

	/* Check for possible option */
	for(i=1; i<argc; i++) if ( argv[i][0] == '-' )
	{
		if (strcmp(argv[i], "-port") == 0)
		{
			if (i+1 < argc) port = atoi(argv[i+1]);
		}
      else 
      {
         usage( argv[0] );
         exit(0);
      }
	}

	if( (sid = EXPORT(port)) < 0 ) {
		nc_perror("EXPORT");
		exit( 1 );
	}

	if( (status = rwait( 0, 0 )) < 0 ) {
		nc_perror("rwait");
		exit( 1 );
	}
}


/* These are simple test functions for simple vaiables.
 * They all have one 'in' and one 'ref out' parameter of
 * the same type. The 'out' parameter contains the 'in' parameter 
 * after a simple operation (usually multiply by two).
 * The returned function value is the value of the 'in' parameter.
 */  

/* unsigned char = NC "char"
 *    8 bits ASCII character
 */
unsigned char char2( chi, cho)
   char chi, *cho;
{
   *cho = 2 * chi; return ( chi );
}

/* unsigned char = NC "boolean"
 *    8 bits 
 */
unsigned char charx( chi, cho )
   char chi, *cho;
{
   *cho = !chi; return ( chi );
}

/* short int = NC "short integer"
 *    16 bits 
 */
short int short2( si, so )
   short si, *so;
{
   *so = 2 * si; return ( si );
} 
 
/* long int = NC "long integer"
 *    32 bits
 */
long int long2( li, lo )
   long int li, *lo;
{
   *lo = 2 * li; return ( li );
} 

/* unsigned short int = NC "short cardinal"
 *    16 bits
 */
unsigned short ushort2( usi, uso )
   unsigned short usi, *uso;
{
   *uso = 2 * usi; return ( usi );
}

/* unsigned long int = NC "long cardinal"
 *    32 bits
 */
unsigned long ulong2( uli, ulo )
   unsigned long int uli, *ulo;
{
   *ulo = 2 * uli; return ( uli );
}
 
/* unsigned short int = NC "short unspecified" or "unspecified"
 *    16 bits, no swaps
 */
unsigned short usspx( usspi, usspo )
   unsigned short int usspi, *usspo;
{
   *usspo = USSPI; return ( usspi );
}

/* unsigned long int = NC "long unspecified"
 *    32 bits, no swaps
 */
unsigned long ulspx( ulspi, ulspo )
   unsigned long int ulspi, *ulspo;
{
   *ulspo = ULSPI; return ( ulspi );
} 

/* float = NC "float"
 *    IEEE-Std.754, 32 bits format 
 */
float float2( fi, fo, status )
   float fi;
   float  *fo;
   short  *status;
{  
   double   diff;
   float_U  Ufi; 
   float_U  Uft;

   Ufi.f = fi;
   Uft.f = FNUMB;
/*
      printf("Compare:  %40.17f\t,%40.17f\n",  fi, FNUMB);
   printf("Rec'd pattern=0x%x 0x%x 0x%x 0x%x\n", Ufi.c[0], Ufi.c[1], Ufi.c[2], Ufi.c[3]);
   printf("Burned pattern=0x%x 0x%x 0x%x 0x%x\n", Uft.c[0], Uft.c[1], Uft.c[2], Uft.c[3]);
   printf("size: %d\n", sizeof(float_U));
*/
/*   if (Ufi.f != Uft.f) */
    if (( diff = fabs((double)(fi - FNUMB))) > (1.e-6) )
   {
      printf("Compare failed\n %40.17f\t,%40.17f\n", diff, FNUMB);
      *status = -1;				/* Cannot recognize number */
   }
   *fo = 2 * Ufi.f; return( Ufi.f );
}
 
/* double float = NC "long float"
 *    IEEE-Std.754, 64 bits format
 *    (The test function returns the floating representation of zero
 *     if the 'in' parameter does not make sense.)
*/
double double2( dfi, dfo, status )
   double dfi, *dfo;
   short  *status;
{ 
   double_U  Udfi;
   double_U  Udft;

   Udfi.d = dfi;
   Udft.d = FNUMB;
/*
      printf("Compare:  %40.17f\t,%40.17f\n",  dfi, FNUMB);
   printf("Rec'd pattern=0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n", Udfi.c[0], Udfi.c[1], Udfi.c[2], Udfi.c[3], Udfi.c[4], Udfi.c[5], Udfi.c[6], Udfi.c[7]);
   printf("Burned pattern=0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n", Udft.c[0], Udft.c[1], Udft.c[2], Udft.c[3], Udft.c[4], Udft.c[5], Udft.c[6], Udft.c[7]);
*/

   if (dfi != DNUMB)
		*status = -1;			/* Cannot recognize number. */
   *dfo = 2 * dfi; return( dfi );
}

     


void	usage( argv0 )
	char *argv0;
{
printf( "Usage: %s [-options]\n\
Valid options are (defaults in parenthesis):\n\
\t-port <port>\tspecify tcp/ip service port (nc_tst);\n\
\t-help\t\t provide some help for other commands;\n\
\n", argv0 );
} 
 

