/* %W% %E% */
/*
 * Origin       Eric Cooper UCB
 *
 * 13/02/89 (raw) added code for OS-9
 * 31/05/90 (raw) change time calculation for OS-9
 * 26.06.90 kk  Three types of signal interface are defined:
 *              BSDsignal (sigvec() etc.) SYSV3signal (sigset() etc.)
 *              and SYSVsignal (only rudimentary signal())
 *              All systems should define one of the three
 *              Two types of time interface are used: BSDtime (gettimeofday())
 *              and SYSVtime (time()).
 *
 * LP - 23.05.2000 - ANSI compliant variable declaration
 */

#include "tm_include.h"

#if defined(OS9)

#define TENPOW6 1000000
#define SECPERDAY 86400

static struct {
	struct {
		short grghour;
		unsigned char grgmin;
		unsigned char grgsec;
	} grgtime;
	struct {
		short grgyear;
		unsigned char grgmon;
		unsigned char grgday;
	} grgdate;
} grg70 = {{0,0,0},{1970,1,1}};

typedef struct
{
	long jultime;
	long juldate;
} julcal;

static julcal *pjul70;
static int undef70 = 1;
#endif /* os9 */

/*
 * Get the time value for now.
 */

void tm_now(register struct timeval *t)
{
#if defined(OS9)
  /*
   * LP - variable declarations moved from lower #if defined(OS9)
   * block because ANSI specs!!!
   */
  long jultime;
  long juldate;
  short julday;

  struct {
	  short ticsec;
	  short curtic;
  } jultic;
#endif
  
#if !(defined(SYSVtime) || defined(OS9))
/* BSDtime is default */
	gettimeofday(t, 0);
#else
  
#if defined(SYSVtime)
	time(&t->tv_sec);
	t->tv_usec = 0L;
#endif

#if defined(OS9)
/*
 * The time function of OS-9 returns an entirely different type of
 * variable as the UNIX function; in addition, it is not referenced
 * to beginning of 1970 as in UNIX.
 * Therefore we first have to get the beginning of 1970 in OS-9
 * time variables (once is sufficient), and thence reference every-
 * thing relative to that date.
 */
 
	if (undef70) {
		_julian(&grg70.grgtime,&grg70.grgdate);
		pjul70=(julcal *)(&grg70);
		undef70=0;
	}
	_sysdate(3,&jultime,&juldate,&julday,&jultic);
	t->tv_sec=(juldate - pjul70->juldate)*SECPERDAY + jultime;
	t->tv_usec=jultic.curtic*TENPOW6/jultic.ticsec;
#endif /* os9 */

#endif

}
