/* %W% %E% */
/*
 * Origin       Eric Cooper UCB
 *
 * 13/02/89 (raw) added signal call for os9
 * 29/05/90 (raw) include stdio.h if DEBUG; use "if defined" constructs
 * 31/05/90 (raw) use u-sec information for OS-9
 * 26.06.90 kk  Three types of signal interface are defined:
 *              BSDsignal (sigvec() etc.) SYSV3signal (sigset() etc.)
 *              and SYSVsignal (only rudimentary signal())
 *              All systems should define one of the three
 *              Two types of time interface are used: BSDtime (gettimeofday())
 *              and SYSVtime (time()).
 */

#include <signal.h>
#include "tm_include.h"
#if defined(DEBUG)
#include <stdio.h>
#endif
extern  int tm_enabled;

/*
 * Start a timer. The interval itself is taken from the timer structure.
 * Note that interrupts must be masked when this is called.
 */

void
tm_start(register struct timer *t)
{
	struct timeval now;
	static int pid = -1;

	ASSERT(t->link != t);

	/* do not start the timer if the interval is zero */
	if (t->interval.tv_sec == 0L && t->interval.tv_usec == 0L)
		return;

	/* enable SIGALRM interrupt if not enabled */
	if( !tm_enabled ) {
#if defined (_POSIX_SOURCE)
	struct sigaction sa;       ;

	sa.sa_handler = (void (*)())tm_interrupt;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	sigaddset(&sa.sa_mask, SIGALRM);
	sigaddset(&sa.sa_mask, SIGIO);
	if ((sigaction (SIGALRM, &sa, 0)) != 0) {
		perror ("sigaction ()");
		exit (1);
	}
#else
#if defined(BSDsignal)
		struct sigvec sv;

		sv.sv_handler = (void (*)())tm_interrupt;
		sv.sv_mask = TM_INTERRUPTMASK;
		sv.sv_onstack = 0;
		if (sigvec(SIGALRM, &sv, 0) != 0) {
			perror("sigvec");
			exit(1);
		}
#endif /* BSDsignal */

#if defined(SYSV3signal)
		sigset(SIGALRM, tm_interrupt);
#endif

#if defined(SYSVsignal)
		/*signal(SIGALRM, (void*)tm_interrupt);*/
		signal(SIGALRM, tm_interrupt);
#endif
#endif /* _POSIX_SOURCE */

		tm_enabled = 1;
	}
	/*
	 * Compute absolute deadline: time now plus specified interval.
	 */
	tm_now(&now);
	if (!q_onqueue(&tm_queue, (struct queue_item *)t)) {
		t->deadline.tv_sec = now.tv_sec + t->interval.tv_sec;
#if !defined(SYSVtime)
		if ((t->deadline.tv_usec = now.tv_usec + t->interval.tv_usec) >= 1000000) {
			t->deadline.tv_usec -= 1000000;
			t->deadline.tv_sec += 1;
		}
#else
		t->deadline.tv_usec = 0L;
#endif
		q_insert(&tm_queue, tm_timertest, (struct queue_item *)t, (queue_args)0);
	}
	t = (struct timer *) q_find(&tm_queue, tm_deadline, (queue_args)&now);
	if (t != ZERO) {
		/*
		 * Found a timer to start.
		 */
		tm_startclock(&now, &t->deadline);
	} else {
		/*
		 * Timers have all expired.
		 * Cause an interrupt at the next opportunity.
		 */
		if (pid == -1)
			pid = getpid();
		kill(pid, SIGALRM);
	}
	TRACE(tm_tracetimers && tm_printtimers("tm_start"));
}
