/*
 * usage: youok host port [code]
 *
 * code = s : get statistics
 *
 */

#include <stdio.h>
#include "sysconfig.h"
#include "errtype.h"
#include "ncc.h"

#define EVERYONE 0 /* Dummy parameter - stat does not use it yet */

int main(argc, argv)
  int argc;
  char **argv;
{
  int sid;
  int port;
  long int timeout = -1;
  long int status;
  long int call_total;
  nc_statistics_t st;

  if ((argc < 3) || (argc > 4))
  {
    fprintf(stderr, "Usage: %s machine port [s]\n", argv[0]);
    exit(1);
  }

  if ((argc == 4) && (*argv[3] != 'e')  && (*argv[3] != 's') && (*argv[3] != 't'))
  {
    fprintf(stderr, "Usage: %s machine port [s]\n", argv[0]);
    exit (1);
  }
  port = atoi(argv[2]);

  if ((argc == 4) && (*argv[3] == 't'))
    nc_trace("mr");
  if ((sid = import_ncr(argv[1], port)) < 0 )
  {
    nc_perror("import_ncr()");
    exit(1);
  }

  if (timeout != -1)
    nc_calltmo(sid, timeout);

  /* Just check if the server is OK */
  if(argc==3){
  if (ncr_util(NCR_ARE_YOU_OK) < 0)
    {
      nc_perror("ncr_util()");
      printf("RPC to %s on port %d failed\n", argv[1], port);
    } 
  else 
    {
      printf("server on host %s port %d replied OK\n", argv[1], port);
    }
  }
  else{
  switch (*argv[3])
    {
      case 'e':
        if (ncr_util(NCR_EXIT) < 0)
        {
          nc_perror("ncr_util()");
          printf("RPC to %s on port %d failed\n", argv[1], port);
        }
        else 
        {
          printf("OK\n");
        }
        break;
      case 's':
        /* Get statistics from the server */
	if ((status = ncr_nc_stat (EVERYONE, &st)) != 0)
        {
	  nc_perror ("ncr_nc_stat ()");
          exit (1);
        }
        call_total = st.call_type[1] + st.call_type[3] + st.call_type[4] + st.call_type[5];
        printf("\n");
        printf ("  Statistics from %s port %d\n", argv[1], port);
        printf (" ------------------------------------\n");
        printf ("  Import : %23d\n", st.import);
        printf ("  Export : %23d\n", st.export);
        printf ("  Forks : %24d\n", st.forks);
        printf ("  Client calls total : %11d\n", st.cl_calls);
        printf ("  Client send length : %11d\n", st.cl_sendlen);
        printf ("  Client receive length : %8d\n", st.cl_receivelen);
        printf ("  Server calls total : %11d\n", call_total);
        printf ("   by blocking call : %12d\n",st.call_type[1]);
        printf ("   by message : %18d\n",st.call_type[3]);
        printf ("   by no-reply call : %12d\n",st.call_type[4]);
        printf ("   by unreliable call : %10d\n",st.call_type[5]);
        printf ("  Return messages : %14d\n",st.call_type[2]);
        printf ("  Server receive length : %8d\n", st.sr_receivelen);
        printf ("  Server send length : %11d\n", st.sr_sendlen);
        printf ("\n");
        break;
      default:
        break;
    }
}
}


